# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable
from datetime import date

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.qbd import purchase_order_list_params, purchase_order_create_params, purchase_order_update_params
from ...pagination import SyncCursorPage, AsyncCursorPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.qbd.purchase_order import PurchaseOrder
from ...types.qbd.purchase_order_delete_response import PurchaseOrderDeleteResponse

__all__ = ["PurchaseOrdersResource", "AsyncPurchaseOrdersResource"]


class PurchaseOrdersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> PurchaseOrdersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#accessing-raw-response-data-eg-headers
        """
        return PurchaseOrdersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PurchaseOrdersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#with_streaming_response
        """
        return PurchaseOrdersResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        transaction_date: Union[str, date],
        conductor_end_user_id: str,
        class_id: str | NotGiven = NOT_GIVEN,
        document_template_id: str | NotGiven = NOT_GIVEN,
        due_date: Union[str, date] | NotGiven = NOT_GIVEN,
        exchange_rate: float | NotGiven = NOT_GIVEN,
        expected_date: Union[str, date] | NotGiven = NOT_GIVEN,
        external_id: str | NotGiven = NOT_GIVEN,
        inventory_site_id: str | NotGiven = NOT_GIVEN,
        is_queued_for_email: bool | NotGiven = NOT_GIVEN,
        is_queued_for_print: bool | NotGiven = NOT_GIVEN,
        line_groups: Iterable[purchase_order_create_params.LineGroup] | NotGiven = NOT_GIVEN,
        lines: Iterable[purchase_order_create_params.Line] | NotGiven = NOT_GIVEN,
        memo: str | NotGiven = NOT_GIVEN,
        other_custom_field1: str | NotGiven = NOT_GIVEN,
        other_custom_field2: str | NotGiven = NOT_GIVEN,
        ref_number: str | NotGiven = NOT_GIVEN,
        sales_tax_code_id: str | NotGiven = NOT_GIVEN,
        shipment_origin: str | NotGiven = NOT_GIVEN,
        shipping_address: purchase_order_create_params.ShippingAddress | NotGiven = NOT_GIVEN,
        shipping_method_id: str | NotGiven = NOT_GIVEN,
        ship_to_entity_id: str | NotGiven = NOT_GIVEN,
        terms_id: str | NotGiven = NOT_GIVEN,
        vendor_address: purchase_order_create_params.VendorAddress | NotGiven = NOT_GIVEN,
        vendor_id: str | NotGiven = NOT_GIVEN,
        vendor_message: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PurchaseOrder:
        """
        Creates a new purchase order.

        Args:
          transaction_date: The date of this purchase order, in ISO 8601 format (YYYY-MM-DD).

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          class_id: The purchase order's class. Classes can be used to categorize objects into
              meaningful segments, such as department, location, or type of work. In
              QuickBooks, class tracking is off by default. A class defined here is
              automatically used in this purchase order's line items unless overridden at the
              line item level.

          document_template_id: The predefined template in QuickBooks that determines the layout and formatting
              for this purchase order when printed or displayed.

          due_date: The date by which this purchase order must be paid, in ISO 8601 format
              (YYYY-MM-DD).

          exchange_rate: The market exchange rate between this purchase order's currency and the home
              currency in QuickBooks at the time of this transaction. Represented as a decimal
              value (e.g., 1.2345 for 1 EUR = 1.2345 USD if USD is the home currency).

          expected_date: The date on which shipment of this purchase order is expected to be completed,
              in ISO 8601 format (YYYY-MM-DD).

          external_id: A globally unique identifier (GUID) you, the developer, can provide for tracking
              this object in your external system. This field is immutable and can only be set
              during object creation.

              **IMPORTANT**: This field must be formatted as a valid GUID; otherwise,
              QuickBooks will return an error.

          inventory_site_id: The site location where inventory for the item associated with this purchase
              order is stored.

          is_queued_for_email: Indicates whether this purchase order is included in the queue of documents for
              QuickBooks to email to the customer.

          is_queued_for_print: Indicates whether this purchase order is included in the queue of documents for
              QuickBooks to print.

          line_groups: The purchase order's line item groups, each representing a predefined set of
              related items.

              **IMPORTANT**: You must specify `lines`, `lineGroups`, or both when creating a
              purchase order.

          lines: The purchase order's line items, each representing a single product or service
              ordered.

              **IMPORTANT**: You must specify `lines`, `lineGroups`, or both when creating a
              purchase order.

          memo: A memo or note for this purchase order that appears in reports, but not on the
              purchase order.

          other_custom_field1: A built-in custom field for additional information specific to this purchase
              order. Unlike the user-defined fields in the `customFields` array, this is a
              standard QuickBooks field that exists for all purchase orders for convenience.
              Developers often use this field for tracking information that doesn't fit into
              other standard QuickBooks fields. Hidden by default in the QuickBooks UI.

          other_custom_field2: A second built-in custom field for additional information specific to this
              purchase order. Unlike the user-defined fields in the `customFields` array, this
              is a standard QuickBooks field that exists for all purchase orders for
              convenience. Like `otherCustomField1`, developers often use this field for
              tracking information that doesn't fit into other standard QuickBooks fields.
              Hidden by default in the QuickBooks UI.

          ref_number: The case-sensitive user-defined reference number for this purchase order, which
              can be used to identify the transaction in QuickBooks. This value is not
              required to be unique and can be arbitrarily changed by the QuickBooks user.
              When left blank in this create request, this field will be left blank in
              QuickBooks (i.e., it does _not_ auto-increment).

          sales_tax_code_id: The sales-tax code for this purchase order, determining whether it is taxable or
              non-taxable. If set, this overrides any sales-tax codes defined on the vendor.
              This can be overridden on the purchase order's individual lines.

              Default codes include "Non" (non-taxable) and "Tax" (taxable), but custom codes
              can also be created in QuickBooks. If QuickBooks is not set up to charge sales
              tax (via the "Do You Charge Sales Tax?" preference), it will assign the default
              non-taxable code to all sales.

          shipment_origin: The origin location from where the product associated with this purchase order
              is shipped. This is the point at which ownership and liability for goods
              transfer from seller to buyer. Internally, QuickBooks uses the term "FOB" for
              this field, which stands for "freight on board". This field is informational and
              has no accounting implications.

          shipping_address: The purchase order's shipping address.

          shipping_method_id: The shipping method used for this purchase order, such as standard mail or
              overnight delivery.

          ship_to_entity_id: The customer, vendor, employee, or other entity to whom this purchase order is
              to be shipped.

          terms_id: The purchase order's payment terms, defining when payment is due and any
              applicable discounts.

          vendor_address: The address of the vendor who sent this purchase order.

          vendor_id: The vendor who sent this purchase order for goods or services purchased.

          vendor_message: A message to be printed on this purchase order for the vendor to read.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._post(
            "/quickbooks-desktop/purchase-orders",
            body=maybe_transform(
                {
                    "transaction_date": transaction_date,
                    "class_id": class_id,
                    "document_template_id": document_template_id,
                    "due_date": due_date,
                    "exchange_rate": exchange_rate,
                    "expected_date": expected_date,
                    "external_id": external_id,
                    "inventory_site_id": inventory_site_id,
                    "is_queued_for_email": is_queued_for_email,
                    "is_queued_for_print": is_queued_for_print,
                    "line_groups": line_groups,
                    "lines": lines,
                    "memo": memo,
                    "other_custom_field1": other_custom_field1,
                    "other_custom_field2": other_custom_field2,
                    "ref_number": ref_number,
                    "sales_tax_code_id": sales_tax_code_id,
                    "shipment_origin": shipment_origin,
                    "shipping_address": shipping_address,
                    "shipping_method_id": shipping_method_id,
                    "ship_to_entity_id": ship_to_entity_id,
                    "terms_id": terms_id,
                    "vendor_address": vendor_address,
                    "vendor_id": vendor_id,
                    "vendor_message": vendor_message,
                },
                purchase_order_create_params.PurchaseOrderCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PurchaseOrder,
        )

    def retrieve(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PurchaseOrder:
        """
        Retrieves a purchase order by ID.

        Args:
          id: The QuickBooks-assigned unique identifier of the purchase order to retrieve.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get(
            f"/quickbooks-desktop/purchase-orders/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PurchaseOrder,
        )

    def update(
        self,
        id: str,
        *,
        revision_number: str,
        conductor_end_user_id: str,
        class_id: str | NotGiven = NOT_GIVEN,
        document_template_id: str | NotGiven = NOT_GIVEN,
        due_date: Union[str, date] | NotGiven = NOT_GIVEN,
        exchange_rate: float | NotGiven = NOT_GIVEN,
        expected_date: Union[str, date] | NotGiven = NOT_GIVEN,
        inventory_site_id: str | NotGiven = NOT_GIVEN,
        is_manually_closed: bool | NotGiven = NOT_GIVEN,
        is_queued_for_email: bool | NotGiven = NOT_GIVEN,
        is_queued_for_print: bool | NotGiven = NOT_GIVEN,
        line_groups: Iterable[purchase_order_update_params.LineGroup] | NotGiven = NOT_GIVEN,
        lines: Iterable[purchase_order_update_params.Line] | NotGiven = NOT_GIVEN,
        memo: str | NotGiven = NOT_GIVEN,
        other_custom_field1: str | NotGiven = NOT_GIVEN,
        other_custom_field2: str | NotGiven = NOT_GIVEN,
        ref_number: str | NotGiven = NOT_GIVEN,
        sales_tax_code_id: str | NotGiven = NOT_GIVEN,
        shipment_origin: str | NotGiven = NOT_GIVEN,
        shipping_address: purchase_order_update_params.ShippingAddress | NotGiven = NOT_GIVEN,
        shipping_method_id: str | NotGiven = NOT_GIVEN,
        ship_to_entity_id: str | NotGiven = NOT_GIVEN,
        terms_id: str | NotGiven = NOT_GIVEN,
        transaction_date: Union[str, date] | NotGiven = NOT_GIVEN,
        vendor_address: purchase_order_update_params.VendorAddress | NotGiven = NOT_GIVEN,
        vendor_id: str | NotGiven = NOT_GIVEN,
        vendor_message: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PurchaseOrder:
        """
        Updates an existing purchase order.

        Args:
          id: The QuickBooks-assigned unique identifier of the purchase order to update.

          revision_number: The current QuickBooks-assigned revision number of the purchase order object you
              are updating, which you can get by fetching the object first. Provide the most
              recent `revisionNumber` to ensure you're working with the latest data;
              otherwise, the update will return an error.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          class_id: The purchase order's class. Classes can be used to categorize objects into
              meaningful segments, such as department, location, or type of work. In
              QuickBooks, class tracking is off by default. A class defined here is
              automatically used in this purchase order's line items unless overridden at the
              line item level.

          document_template_id: The predefined template in QuickBooks that determines the layout and formatting
              for this purchase order when printed or displayed.

          due_date: The date by which this purchase order must be paid, in ISO 8601 format
              (YYYY-MM-DD).

          exchange_rate: The market exchange rate between this purchase order's currency and the home
              currency in QuickBooks at the time of this transaction. Represented as a decimal
              value (e.g., 1.2345 for 1 EUR = 1.2345 USD if USD is the home currency).

          expected_date: The date on which shipment of this purchase order is expected to be completed,
              in ISO 8601 format (YYYY-MM-DD).

          inventory_site_id: The site location where inventory for the item associated with this purchase
              order is stored.

          is_manually_closed: Indicates whether this purchase order has been manually marked as closed, even
              if all items have not been received or the sale has not been cancelled. Once the
              purchase order is marked as closed, all of its line items become closed as well.
              You cannot change `isManuallyClosed` to `false` after the purchase order has
              been fully received.

          is_queued_for_email: Indicates whether this purchase order is included in the queue of documents for
              QuickBooks to email to the customer.

          is_queued_for_print: Indicates whether this purchase order is included in the queue of documents for
              QuickBooks to print.

          line_groups: The purchase order's line item groups, each representing a predefined set of
              related items.

              **IMPORTANT**:

              1. Including this array in your update request will **REPLACE** all existing
                 line item groups for the purchase order with this array. To keep any existing
                 line item groups, you must include them in this array even if they have not
                 changed. **Any line item groups not included will be removed.**

              2. To add a new line item group, include it here with the `id` field set to
                 `-1`.

              3. If you do not wish to modify any line item groups, omit this field entirely
                 to keep them unchanged.

          lines: The purchase order's line items, each representing a single product or service
              ordered.

              **IMPORTANT**:

              1. Including this array in your update request will **REPLACE** all existing
                 line items for the purchase order with this array. To keep any existing line
                 items, you must include them in this array even if they have not changed.
                 **Any line items not included will be removed.**

              2. To add a new line item, include it here with the `id` field set to `-1`.

              3. If you do not wish to modify any line items, omit this field entirely to keep
                 them unchanged.

          memo: A memo or note for this purchase order that appears in reports, but not on the
              purchase order.

          other_custom_field1: A built-in custom field for additional information specific to this purchase
              order. Unlike the user-defined fields in the `customFields` array, this is a
              standard QuickBooks field that exists for all purchase orders for convenience.
              Developers often use this field for tracking information that doesn't fit into
              other standard QuickBooks fields. Hidden by default in the QuickBooks UI.

          other_custom_field2: A second built-in custom field for additional information specific to this
              purchase order. Unlike the user-defined fields in the `customFields` array, this
              is a standard QuickBooks field that exists for all purchase orders for
              convenience. Like `otherCustomField1`, developers often use this field for
              tracking information that doesn't fit into other standard QuickBooks fields.
              Hidden by default in the QuickBooks UI.

          ref_number: The case-sensitive user-defined reference number for this purchase order, which
              can be used to identify the transaction in QuickBooks. This value is not
              required to be unique and can be arbitrarily changed by the QuickBooks user.

          sales_tax_code_id: The sales-tax code for this purchase order, determining whether it is taxable or
              non-taxable. If set, this overrides any sales-tax codes defined on the vendor.
              This can be overridden on the purchase order's individual lines.

              Default codes include "Non" (non-taxable) and "Tax" (taxable), but custom codes
              can also be created in QuickBooks. If QuickBooks is not set up to charge sales
              tax (via the "Do You Charge Sales Tax?" preference), it will assign the default
              non-taxable code to all sales.

          shipment_origin: The origin location from where the product associated with this purchase order
              is shipped. This is the point at which ownership and liability for goods
              transfer from seller to buyer. Internally, QuickBooks uses the term "FOB" for
              this field, which stands for "freight on board". This field is informational and
              has no accounting implications.

          shipping_address: The purchase order's shipping address.

          shipping_method_id: The shipping method used for this purchase order, such as standard mail or
              overnight delivery.

          ship_to_entity_id: The customer, vendor, employee, or other entity to whom this purchase order is
              to be shipped.

          terms_id: The purchase order's payment terms, defining when payment is due and any
              applicable discounts.

          transaction_date: The date of this purchase order, in ISO 8601 format (YYYY-MM-DD).

          vendor_address: The address of the vendor who sent this purchase order.

          vendor_id: The vendor who sent this purchase order for goods or services purchased.

          vendor_message: A message to be printed on this purchase order for the vendor to read.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._post(
            f"/quickbooks-desktop/purchase-orders/{id}",
            body=maybe_transform(
                {
                    "revision_number": revision_number,
                    "class_id": class_id,
                    "document_template_id": document_template_id,
                    "due_date": due_date,
                    "exchange_rate": exchange_rate,
                    "expected_date": expected_date,
                    "inventory_site_id": inventory_site_id,
                    "is_manually_closed": is_manually_closed,
                    "is_queued_for_email": is_queued_for_email,
                    "is_queued_for_print": is_queued_for_print,
                    "line_groups": line_groups,
                    "lines": lines,
                    "memo": memo,
                    "other_custom_field1": other_custom_field1,
                    "other_custom_field2": other_custom_field2,
                    "ref_number": ref_number,
                    "sales_tax_code_id": sales_tax_code_id,
                    "shipment_origin": shipment_origin,
                    "shipping_address": shipping_address,
                    "shipping_method_id": shipping_method_id,
                    "ship_to_entity_id": ship_to_entity_id,
                    "terms_id": terms_id,
                    "transaction_date": transaction_date,
                    "vendor_address": vendor_address,
                    "vendor_id": vendor_id,
                    "vendor_message": vendor_message,
                },
                purchase_order_update_params.PurchaseOrderUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PurchaseOrder,
        )

    def list(
        self,
        *,
        conductor_end_user_id: str,
        account_ids: List[str] | NotGiven = NOT_GIVEN,
        currency_ids: List[str] | NotGiven = NOT_GIVEN,
        cursor: str | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        include_line_items: bool | NotGiven = NOT_GIVEN,
        include_linked_transactions: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        ref_number_contains: str | NotGiven = NOT_GIVEN,
        ref_number_ends_with: str | NotGiven = NOT_GIVEN,
        ref_number_from: str | NotGiven = NOT_GIVEN,
        ref_numbers: List[str] | NotGiven = NOT_GIVEN,
        ref_number_starts_with: str | NotGiven = NOT_GIVEN,
        ref_number_to: str | NotGiven = NOT_GIVEN,
        transaction_date_from: Union[str, date] | NotGiven = NOT_GIVEN,
        transaction_date_to: Union[str, date] | NotGiven = NOT_GIVEN,
        updated_after: str | NotGiven = NOT_GIVEN,
        updated_before: str | NotGiven = NOT_GIVEN,
        vendor_ids: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursorPage[PurchaseOrder]:
        """Returns a list of purchase orders.

        Use the `cursor` parameter to paginate
        through the results.

        Args:
          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          account_ids: Filter for purchase orders associated with these accounts.

          currency_ids: Filter for purchase orders in these currencies.

          cursor: The pagination token to fetch the next set of results when paginating with the
              `limit` parameter. Do not include this parameter on the first call. Use the
              `nextCursor` value returned in the previous response to request subsequent
              results.

          ids: Filter for specific purchase orders by their QuickBooks-assigned unique
              identifier(s).

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          include_line_items: Whether to include line items in the response. Defaults to `true`.

          include_linked_transactions: Whether to include linked transactions in the response. Defaults to `false`. For
              example, a payment linked to the corresponding purchase order.

          limit: The maximum number of objects to return. Accepts values ranging from 1 to 150,
              defaults to 150. When used with cursor-based pagination, this parameter controls
              how many results are returned per page. To paginate through results, combine
              this with the `cursor` parameter. Each response will include a `nextCursor`
              value that can be passed to subsequent requests to retrieve the next page of
              results.

          ref_number_contains: Filter for purchase orders whose `refNumber` contains this substring. NOTE: If
              you use this parameter, you cannot also use `refNumberStartsWith` or
              `refNumberEndsWith`.

          ref_number_ends_with: Filter for purchase orders whose `refNumber` ends with this substring. NOTE: If
              you use this parameter, you cannot also use `refNumberContains` or
              `refNumberStartsWith`.

          ref_number_from: Filter for purchase orders whose `refNumber` is greater than or equal to this
              value. If omitted, the range will begin with the first number of the list. Uses
              a numerical comparison for values that contain only digits; otherwise, uses a
              lexicographical comparison.

          ref_numbers: Filter for specific purchase orders by their ref-number(s), case-sensitive. In
              QuickBooks, ref-numbers are not required to be unique and can be arbitrarily
              changed by the QuickBooks user.

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          ref_number_starts_with:
              Filter for purchase orders whose `refNumber` starts with this substring. NOTE:
              If you use this parameter, you cannot also use `refNumberContains` or
              `refNumberEndsWith`.

          ref_number_to: Filter for purchase orders whose `refNumber` is less than or equal to this
              value. If omitted, the range will end with the last number of the list. Uses a
              numerical comparison for values that contain only digits; otherwise, uses a
              lexicographical comparison.

          transaction_date_from: Filter for purchase orders whose `date` field is on or after this date, in ISO
              8601 format (YYYY-MM-DD).

          transaction_date_to: Filter for purchase orders whose `date` field is on or before this date, in ISO
              8601 format (YYYY-MM-DD).

          updated_after: Filter for purchase orders updated on or after this date and time, in ISO 8601
              format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time
              is assumed to be 00:00:00 of that day.

          updated_before: Filter for purchase orders updated on or before this date and time, in ISO 8601
              format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time
              is assumed to be 23:59:59 of that day.

          vendor_ids: Filter for purchase orders sent to these vendors.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get_api_list(
            "/quickbooks-desktop/purchase-orders",
            page=SyncCursorPage[PurchaseOrder],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_ids": account_ids,
                        "currency_ids": currency_ids,
                        "cursor": cursor,
                        "ids": ids,
                        "include_line_items": include_line_items,
                        "include_linked_transactions": include_linked_transactions,
                        "limit": limit,
                        "ref_number_contains": ref_number_contains,
                        "ref_number_ends_with": ref_number_ends_with,
                        "ref_number_from": ref_number_from,
                        "ref_numbers": ref_numbers,
                        "ref_number_starts_with": ref_number_starts_with,
                        "ref_number_to": ref_number_to,
                        "transaction_date_from": transaction_date_from,
                        "transaction_date_to": transaction_date_to,
                        "updated_after": updated_after,
                        "updated_before": updated_before,
                        "vendor_ids": vendor_ids,
                    },
                    purchase_order_list_params.PurchaseOrderListParams,
                ),
            ),
            model=PurchaseOrder,
        )

    def delete(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PurchaseOrderDeleteResponse:
        """Permanently deletes a a purchase order.

        The deletion will fail if the purchase
        order is currently in use or has any linked transactions that are in use.

        Args:
          id: The QuickBooks-assigned unique identifier of the purchase order to delete.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._delete(
            f"/quickbooks-desktop/purchase-orders/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PurchaseOrderDeleteResponse,
        )


class AsyncPurchaseOrdersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncPurchaseOrdersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPurchaseOrdersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPurchaseOrdersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#with_streaming_response
        """
        return AsyncPurchaseOrdersResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        transaction_date: Union[str, date],
        conductor_end_user_id: str,
        class_id: str | NotGiven = NOT_GIVEN,
        document_template_id: str | NotGiven = NOT_GIVEN,
        due_date: Union[str, date] | NotGiven = NOT_GIVEN,
        exchange_rate: float | NotGiven = NOT_GIVEN,
        expected_date: Union[str, date] | NotGiven = NOT_GIVEN,
        external_id: str | NotGiven = NOT_GIVEN,
        inventory_site_id: str | NotGiven = NOT_GIVEN,
        is_queued_for_email: bool | NotGiven = NOT_GIVEN,
        is_queued_for_print: bool | NotGiven = NOT_GIVEN,
        line_groups: Iterable[purchase_order_create_params.LineGroup] | NotGiven = NOT_GIVEN,
        lines: Iterable[purchase_order_create_params.Line] | NotGiven = NOT_GIVEN,
        memo: str | NotGiven = NOT_GIVEN,
        other_custom_field1: str | NotGiven = NOT_GIVEN,
        other_custom_field2: str | NotGiven = NOT_GIVEN,
        ref_number: str | NotGiven = NOT_GIVEN,
        sales_tax_code_id: str | NotGiven = NOT_GIVEN,
        shipment_origin: str | NotGiven = NOT_GIVEN,
        shipping_address: purchase_order_create_params.ShippingAddress | NotGiven = NOT_GIVEN,
        shipping_method_id: str | NotGiven = NOT_GIVEN,
        ship_to_entity_id: str | NotGiven = NOT_GIVEN,
        terms_id: str | NotGiven = NOT_GIVEN,
        vendor_address: purchase_order_create_params.VendorAddress | NotGiven = NOT_GIVEN,
        vendor_id: str | NotGiven = NOT_GIVEN,
        vendor_message: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PurchaseOrder:
        """
        Creates a new purchase order.

        Args:
          transaction_date: The date of this purchase order, in ISO 8601 format (YYYY-MM-DD).

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          class_id: The purchase order's class. Classes can be used to categorize objects into
              meaningful segments, such as department, location, or type of work. In
              QuickBooks, class tracking is off by default. A class defined here is
              automatically used in this purchase order's line items unless overridden at the
              line item level.

          document_template_id: The predefined template in QuickBooks that determines the layout and formatting
              for this purchase order when printed or displayed.

          due_date: The date by which this purchase order must be paid, in ISO 8601 format
              (YYYY-MM-DD).

          exchange_rate: The market exchange rate between this purchase order's currency and the home
              currency in QuickBooks at the time of this transaction. Represented as a decimal
              value (e.g., 1.2345 for 1 EUR = 1.2345 USD if USD is the home currency).

          expected_date: The date on which shipment of this purchase order is expected to be completed,
              in ISO 8601 format (YYYY-MM-DD).

          external_id: A globally unique identifier (GUID) you, the developer, can provide for tracking
              this object in your external system. This field is immutable and can only be set
              during object creation.

              **IMPORTANT**: This field must be formatted as a valid GUID; otherwise,
              QuickBooks will return an error.

          inventory_site_id: The site location where inventory for the item associated with this purchase
              order is stored.

          is_queued_for_email: Indicates whether this purchase order is included in the queue of documents for
              QuickBooks to email to the customer.

          is_queued_for_print: Indicates whether this purchase order is included in the queue of documents for
              QuickBooks to print.

          line_groups: The purchase order's line item groups, each representing a predefined set of
              related items.

              **IMPORTANT**: You must specify `lines`, `lineGroups`, or both when creating a
              purchase order.

          lines: The purchase order's line items, each representing a single product or service
              ordered.

              **IMPORTANT**: You must specify `lines`, `lineGroups`, or both when creating a
              purchase order.

          memo: A memo or note for this purchase order that appears in reports, but not on the
              purchase order.

          other_custom_field1: A built-in custom field for additional information specific to this purchase
              order. Unlike the user-defined fields in the `customFields` array, this is a
              standard QuickBooks field that exists for all purchase orders for convenience.
              Developers often use this field for tracking information that doesn't fit into
              other standard QuickBooks fields. Hidden by default in the QuickBooks UI.

          other_custom_field2: A second built-in custom field for additional information specific to this
              purchase order. Unlike the user-defined fields in the `customFields` array, this
              is a standard QuickBooks field that exists for all purchase orders for
              convenience. Like `otherCustomField1`, developers often use this field for
              tracking information that doesn't fit into other standard QuickBooks fields.
              Hidden by default in the QuickBooks UI.

          ref_number: The case-sensitive user-defined reference number for this purchase order, which
              can be used to identify the transaction in QuickBooks. This value is not
              required to be unique and can be arbitrarily changed by the QuickBooks user.
              When left blank in this create request, this field will be left blank in
              QuickBooks (i.e., it does _not_ auto-increment).

          sales_tax_code_id: The sales-tax code for this purchase order, determining whether it is taxable or
              non-taxable. If set, this overrides any sales-tax codes defined on the vendor.
              This can be overridden on the purchase order's individual lines.

              Default codes include "Non" (non-taxable) and "Tax" (taxable), but custom codes
              can also be created in QuickBooks. If QuickBooks is not set up to charge sales
              tax (via the "Do You Charge Sales Tax?" preference), it will assign the default
              non-taxable code to all sales.

          shipment_origin: The origin location from where the product associated with this purchase order
              is shipped. This is the point at which ownership and liability for goods
              transfer from seller to buyer. Internally, QuickBooks uses the term "FOB" for
              this field, which stands for "freight on board". This field is informational and
              has no accounting implications.

          shipping_address: The purchase order's shipping address.

          shipping_method_id: The shipping method used for this purchase order, such as standard mail or
              overnight delivery.

          ship_to_entity_id: The customer, vendor, employee, or other entity to whom this purchase order is
              to be shipped.

          terms_id: The purchase order's payment terms, defining when payment is due and any
              applicable discounts.

          vendor_address: The address of the vendor who sent this purchase order.

          vendor_id: The vendor who sent this purchase order for goods or services purchased.

          vendor_message: A message to be printed on this purchase order for the vendor to read.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._post(
            "/quickbooks-desktop/purchase-orders",
            body=await async_maybe_transform(
                {
                    "transaction_date": transaction_date,
                    "class_id": class_id,
                    "document_template_id": document_template_id,
                    "due_date": due_date,
                    "exchange_rate": exchange_rate,
                    "expected_date": expected_date,
                    "external_id": external_id,
                    "inventory_site_id": inventory_site_id,
                    "is_queued_for_email": is_queued_for_email,
                    "is_queued_for_print": is_queued_for_print,
                    "line_groups": line_groups,
                    "lines": lines,
                    "memo": memo,
                    "other_custom_field1": other_custom_field1,
                    "other_custom_field2": other_custom_field2,
                    "ref_number": ref_number,
                    "sales_tax_code_id": sales_tax_code_id,
                    "shipment_origin": shipment_origin,
                    "shipping_address": shipping_address,
                    "shipping_method_id": shipping_method_id,
                    "ship_to_entity_id": ship_to_entity_id,
                    "terms_id": terms_id,
                    "vendor_address": vendor_address,
                    "vendor_id": vendor_id,
                    "vendor_message": vendor_message,
                },
                purchase_order_create_params.PurchaseOrderCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PurchaseOrder,
        )

    async def retrieve(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PurchaseOrder:
        """
        Retrieves a purchase order by ID.

        Args:
          id: The QuickBooks-assigned unique identifier of the purchase order to retrieve.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._get(
            f"/quickbooks-desktop/purchase-orders/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PurchaseOrder,
        )

    async def update(
        self,
        id: str,
        *,
        revision_number: str,
        conductor_end_user_id: str,
        class_id: str | NotGiven = NOT_GIVEN,
        document_template_id: str | NotGiven = NOT_GIVEN,
        due_date: Union[str, date] | NotGiven = NOT_GIVEN,
        exchange_rate: float | NotGiven = NOT_GIVEN,
        expected_date: Union[str, date] | NotGiven = NOT_GIVEN,
        inventory_site_id: str | NotGiven = NOT_GIVEN,
        is_manually_closed: bool | NotGiven = NOT_GIVEN,
        is_queued_for_email: bool | NotGiven = NOT_GIVEN,
        is_queued_for_print: bool | NotGiven = NOT_GIVEN,
        line_groups: Iterable[purchase_order_update_params.LineGroup] | NotGiven = NOT_GIVEN,
        lines: Iterable[purchase_order_update_params.Line] | NotGiven = NOT_GIVEN,
        memo: str | NotGiven = NOT_GIVEN,
        other_custom_field1: str | NotGiven = NOT_GIVEN,
        other_custom_field2: str | NotGiven = NOT_GIVEN,
        ref_number: str | NotGiven = NOT_GIVEN,
        sales_tax_code_id: str | NotGiven = NOT_GIVEN,
        shipment_origin: str | NotGiven = NOT_GIVEN,
        shipping_address: purchase_order_update_params.ShippingAddress | NotGiven = NOT_GIVEN,
        shipping_method_id: str | NotGiven = NOT_GIVEN,
        ship_to_entity_id: str | NotGiven = NOT_GIVEN,
        terms_id: str | NotGiven = NOT_GIVEN,
        transaction_date: Union[str, date] | NotGiven = NOT_GIVEN,
        vendor_address: purchase_order_update_params.VendorAddress | NotGiven = NOT_GIVEN,
        vendor_id: str | NotGiven = NOT_GIVEN,
        vendor_message: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PurchaseOrder:
        """
        Updates an existing purchase order.

        Args:
          id: The QuickBooks-assigned unique identifier of the purchase order to update.

          revision_number: The current QuickBooks-assigned revision number of the purchase order object you
              are updating, which you can get by fetching the object first. Provide the most
              recent `revisionNumber` to ensure you're working with the latest data;
              otherwise, the update will return an error.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          class_id: The purchase order's class. Classes can be used to categorize objects into
              meaningful segments, such as department, location, or type of work. In
              QuickBooks, class tracking is off by default. A class defined here is
              automatically used in this purchase order's line items unless overridden at the
              line item level.

          document_template_id: The predefined template in QuickBooks that determines the layout and formatting
              for this purchase order when printed or displayed.

          due_date: The date by which this purchase order must be paid, in ISO 8601 format
              (YYYY-MM-DD).

          exchange_rate: The market exchange rate between this purchase order's currency and the home
              currency in QuickBooks at the time of this transaction. Represented as a decimal
              value (e.g., 1.2345 for 1 EUR = 1.2345 USD if USD is the home currency).

          expected_date: The date on which shipment of this purchase order is expected to be completed,
              in ISO 8601 format (YYYY-MM-DD).

          inventory_site_id: The site location where inventory for the item associated with this purchase
              order is stored.

          is_manually_closed: Indicates whether this purchase order has been manually marked as closed, even
              if all items have not been received or the sale has not been cancelled. Once the
              purchase order is marked as closed, all of its line items become closed as well.
              You cannot change `isManuallyClosed` to `false` after the purchase order has
              been fully received.

          is_queued_for_email: Indicates whether this purchase order is included in the queue of documents for
              QuickBooks to email to the customer.

          is_queued_for_print: Indicates whether this purchase order is included in the queue of documents for
              QuickBooks to print.

          line_groups: The purchase order's line item groups, each representing a predefined set of
              related items.

              **IMPORTANT**:

              1. Including this array in your update request will **REPLACE** all existing
                 line item groups for the purchase order with this array. To keep any existing
                 line item groups, you must include them in this array even if they have not
                 changed. **Any line item groups not included will be removed.**

              2. To add a new line item group, include it here with the `id` field set to
                 `-1`.

              3. If you do not wish to modify any line item groups, omit this field entirely
                 to keep them unchanged.

          lines: The purchase order's line items, each representing a single product or service
              ordered.

              **IMPORTANT**:

              1. Including this array in your update request will **REPLACE** all existing
                 line items for the purchase order with this array. To keep any existing line
                 items, you must include them in this array even if they have not changed.
                 **Any line items not included will be removed.**

              2. To add a new line item, include it here with the `id` field set to `-1`.

              3. If you do not wish to modify any line items, omit this field entirely to keep
                 them unchanged.

          memo: A memo or note for this purchase order that appears in reports, but not on the
              purchase order.

          other_custom_field1: A built-in custom field for additional information specific to this purchase
              order. Unlike the user-defined fields in the `customFields` array, this is a
              standard QuickBooks field that exists for all purchase orders for convenience.
              Developers often use this field for tracking information that doesn't fit into
              other standard QuickBooks fields. Hidden by default in the QuickBooks UI.

          other_custom_field2: A second built-in custom field for additional information specific to this
              purchase order. Unlike the user-defined fields in the `customFields` array, this
              is a standard QuickBooks field that exists for all purchase orders for
              convenience. Like `otherCustomField1`, developers often use this field for
              tracking information that doesn't fit into other standard QuickBooks fields.
              Hidden by default in the QuickBooks UI.

          ref_number: The case-sensitive user-defined reference number for this purchase order, which
              can be used to identify the transaction in QuickBooks. This value is not
              required to be unique and can be arbitrarily changed by the QuickBooks user.

          sales_tax_code_id: The sales-tax code for this purchase order, determining whether it is taxable or
              non-taxable. If set, this overrides any sales-tax codes defined on the vendor.
              This can be overridden on the purchase order's individual lines.

              Default codes include "Non" (non-taxable) and "Tax" (taxable), but custom codes
              can also be created in QuickBooks. If QuickBooks is not set up to charge sales
              tax (via the "Do You Charge Sales Tax?" preference), it will assign the default
              non-taxable code to all sales.

          shipment_origin: The origin location from where the product associated with this purchase order
              is shipped. This is the point at which ownership and liability for goods
              transfer from seller to buyer. Internally, QuickBooks uses the term "FOB" for
              this field, which stands for "freight on board". This field is informational and
              has no accounting implications.

          shipping_address: The purchase order's shipping address.

          shipping_method_id: The shipping method used for this purchase order, such as standard mail or
              overnight delivery.

          ship_to_entity_id: The customer, vendor, employee, or other entity to whom this purchase order is
              to be shipped.

          terms_id: The purchase order's payment terms, defining when payment is due and any
              applicable discounts.

          transaction_date: The date of this purchase order, in ISO 8601 format (YYYY-MM-DD).

          vendor_address: The address of the vendor who sent this purchase order.

          vendor_id: The vendor who sent this purchase order for goods or services purchased.

          vendor_message: A message to be printed on this purchase order for the vendor to read.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._post(
            f"/quickbooks-desktop/purchase-orders/{id}",
            body=await async_maybe_transform(
                {
                    "revision_number": revision_number,
                    "class_id": class_id,
                    "document_template_id": document_template_id,
                    "due_date": due_date,
                    "exchange_rate": exchange_rate,
                    "expected_date": expected_date,
                    "inventory_site_id": inventory_site_id,
                    "is_manually_closed": is_manually_closed,
                    "is_queued_for_email": is_queued_for_email,
                    "is_queued_for_print": is_queued_for_print,
                    "line_groups": line_groups,
                    "lines": lines,
                    "memo": memo,
                    "other_custom_field1": other_custom_field1,
                    "other_custom_field2": other_custom_field2,
                    "ref_number": ref_number,
                    "sales_tax_code_id": sales_tax_code_id,
                    "shipment_origin": shipment_origin,
                    "shipping_address": shipping_address,
                    "shipping_method_id": shipping_method_id,
                    "ship_to_entity_id": ship_to_entity_id,
                    "terms_id": terms_id,
                    "transaction_date": transaction_date,
                    "vendor_address": vendor_address,
                    "vendor_id": vendor_id,
                    "vendor_message": vendor_message,
                },
                purchase_order_update_params.PurchaseOrderUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PurchaseOrder,
        )

    def list(
        self,
        *,
        conductor_end_user_id: str,
        account_ids: List[str] | NotGiven = NOT_GIVEN,
        currency_ids: List[str] | NotGiven = NOT_GIVEN,
        cursor: str | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        include_line_items: bool | NotGiven = NOT_GIVEN,
        include_linked_transactions: bool | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        ref_number_contains: str | NotGiven = NOT_GIVEN,
        ref_number_ends_with: str | NotGiven = NOT_GIVEN,
        ref_number_from: str | NotGiven = NOT_GIVEN,
        ref_numbers: List[str] | NotGiven = NOT_GIVEN,
        ref_number_starts_with: str | NotGiven = NOT_GIVEN,
        ref_number_to: str | NotGiven = NOT_GIVEN,
        transaction_date_from: Union[str, date] | NotGiven = NOT_GIVEN,
        transaction_date_to: Union[str, date] | NotGiven = NOT_GIVEN,
        updated_after: str | NotGiven = NOT_GIVEN,
        updated_before: str | NotGiven = NOT_GIVEN,
        vendor_ids: List[str] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[PurchaseOrder, AsyncCursorPage[PurchaseOrder]]:
        """Returns a list of purchase orders.

        Use the `cursor` parameter to paginate
        through the results.

        Args:
          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          account_ids: Filter for purchase orders associated with these accounts.

          currency_ids: Filter for purchase orders in these currencies.

          cursor: The pagination token to fetch the next set of results when paginating with the
              `limit` parameter. Do not include this parameter on the first call. Use the
              `nextCursor` value returned in the previous response to request subsequent
              results.

          ids: Filter for specific purchase orders by their QuickBooks-assigned unique
              identifier(s).

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          include_line_items: Whether to include line items in the response. Defaults to `true`.

          include_linked_transactions: Whether to include linked transactions in the response. Defaults to `false`. For
              example, a payment linked to the corresponding purchase order.

          limit: The maximum number of objects to return. Accepts values ranging from 1 to 150,
              defaults to 150. When used with cursor-based pagination, this parameter controls
              how many results are returned per page. To paginate through results, combine
              this with the `cursor` parameter. Each response will include a `nextCursor`
              value that can be passed to subsequent requests to retrieve the next page of
              results.

          ref_number_contains: Filter for purchase orders whose `refNumber` contains this substring. NOTE: If
              you use this parameter, you cannot also use `refNumberStartsWith` or
              `refNumberEndsWith`.

          ref_number_ends_with: Filter for purchase orders whose `refNumber` ends with this substring. NOTE: If
              you use this parameter, you cannot also use `refNumberContains` or
              `refNumberStartsWith`.

          ref_number_from: Filter for purchase orders whose `refNumber` is greater than or equal to this
              value. If omitted, the range will begin with the first number of the list. Uses
              a numerical comparison for values that contain only digits; otherwise, uses a
              lexicographical comparison.

          ref_numbers: Filter for specific purchase orders by their ref-number(s), case-sensitive. In
              QuickBooks, ref-numbers are not required to be unique and can be arbitrarily
              changed by the QuickBooks user.

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          ref_number_starts_with:
              Filter for purchase orders whose `refNumber` starts with this substring. NOTE:
              If you use this parameter, you cannot also use `refNumberContains` or
              `refNumberEndsWith`.

          ref_number_to: Filter for purchase orders whose `refNumber` is less than or equal to this
              value. If omitted, the range will end with the last number of the list. Uses a
              numerical comparison for values that contain only digits; otherwise, uses a
              lexicographical comparison.

          transaction_date_from: Filter for purchase orders whose `date` field is on or after this date, in ISO
              8601 format (YYYY-MM-DD).

          transaction_date_to: Filter for purchase orders whose `date` field is on or before this date, in ISO
              8601 format (YYYY-MM-DD).

          updated_after: Filter for purchase orders updated on or after this date and time, in ISO 8601
              format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time
              is assumed to be 00:00:00 of that day.

          updated_before: Filter for purchase orders updated on or before this date and time, in ISO 8601
              format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time
              is assumed to be 23:59:59 of that day.

          vendor_ids: Filter for purchase orders sent to these vendors.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get_api_list(
            "/quickbooks-desktop/purchase-orders",
            page=AsyncCursorPage[PurchaseOrder],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "account_ids": account_ids,
                        "currency_ids": currency_ids,
                        "cursor": cursor,
                        "ids": ids,
                        "include_line_items": include_line_items,
                        "include_linked_transactions": include_linked_transactions,
                        "limit": limit,
                        "ref_number_contains": ref_number_contains,
                        "ref_number_ends_with": ref_number_ends_with,
                        "ref_number_from": ref_number_from,
                        "ref_numbers": ref_numbers,
                        "ref_number_starts_with": ref_number_starts_with,
                        "ref_number_to": ref_number_to,
                        "transaction_date_from": transaction_date_from,
                        "transaction_date_to": transaction_date_to,
                        "updated_after": updated_after,
                        "updated_before": updated_before,
                        "vendor_ids": vendor_ids,
                    },
                    purchase_order_list_params.PurchaseOrderListParams,
                ),
            ),
            model=PurchaseOrder,
        )

    async def delete(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> PurchaseOrderDeleteResponse:
        """Permanently deletes a a purchase order.

        The deletion will fail if the purchase
        order is currently in use or has any linked transactions that are in use.

        Args:
          id: The QuickBooks-assigned unique identifier of the purchase order to delete.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._delete(
            f"/quickbooks-desktop/purchase-orders/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=PurchaseOrderDeleteResponse,
        )


class PurchaseOrdersResourceWithRawResponse:
    def __init__(self, purchase_orders: PurchaseOrdersResource) -> None:
        self._purchase_orders = purchase_orders

        self.create = to_raw_response_wrapper(
            purchase_orders.create,
        )
        self.retrieve = to_raw_response_wrapper(
            purchase_orders.retrieve,
        )
        self.update = to_raw_response_wrapper(
            purchase_orders.update,
        )
        self.list = to_raw_response_wrapper(
            purchase_orders.list,
        )
        self.delete = to_raw_response_wrapper(
            purchase_orders.delete,
        )


class AsyncPurchaseOrdersResourceWithRawResponse:
    def __init__(self, purchase_orders: AsyncPurchaseOrdersResource) -> None:
        self._purchase_orders = purchase_orders

        self.create = async_to_raw_response_wrapper(
            purchase_orders.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            purchase_orders.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            purchase_orders.update,
        )
        self.list = async_to_raw_response_wrapper(
            purchase_orders.list,
        )
        self.delete = async_to_raw_response_wrapper(
            purchase_orders.delete,
        )


class PurchaseOrdersResourceWithStreamingResponse:
    def __init__(self, purchase_orders: PurchaseOrdersResource) -> None:
        self._purchase_orders = purchase_orders

        self.create = to_streamed_response_wrapper(
            purchase_orders.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            purchase_orders.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            purchase_orders.update,
        )
        self.list = to_streamed_response_wrapper(
            purchase_orders.list,
        )
        self.delete = to_streamed_response_wrapper(
            purchase_orders.delete,
        )


class AsyncPurchaseOrdersResourceWithStreamingResponse:
    def __init__(self, purchase_orders: AsyncPurchaseOrdersResource) -> None:
        self._purchase_orders = purchase_orders

        self.create = async_to_streamed_response_wrapper(
            purchase_orders.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            purchase_orders.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            purchase_orders.update,
        )
        self.list = async_to_streamed_response_wrapper(
            purchase_orders.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            purchase_orders.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable
from datetime import date
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import (
    maybe_transform,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...types.qbd import (
    inventory_assembly_item_list_params,
    inventory_assembly_item_create_params,
    inventory_assembly_item_update_params,
)
from ...pagination import SyncCursorPage, AsyncCursorPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.qbd.inventory_assembly_item import InventoryAssemblyItem

__all__ = ["InventoryAssemblyItemsResource", "AsyncInventoryAssemblyItemsResource"]


class InventoryAssemblyItemsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> InventoryAssemblyItemsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#accessing-raw-response-data-eg-headers
        """
        return InventoryAssemblyItemsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InventoryAssemblyItemsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#with_streaming_response
        """
        return InventoryAssemblyItemsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        asset_account_id: str,
        cogs_account_id: str,
        income_account_id: str,
        name: str,
        conductor_end_user_id: str,
        barcode: inventory_assembly_item_create_params.Barcode | NotGiven = NOT_GIVEN,
        build_notification_threshold: float | NotGiven = NOT_GIVEN,
        class_id: str | NotGiven = NOT_GIVEN,
        external_id: str | NotGiven = NOT_GIVEN,
        inventory_date: Union[str, date] | NotGiven = NOT_GIVEN,
        is_active: bool | NotGiven = NOT_GIVEN,
        lines: Iterable[inventory_assembly_item_create_params.Line] | NotGiven = NOT_GIVEN,
        maximum_quantity_on_hand: float | NotGiven = NOT_GIVEN,
        parent_id: str | NotGiven = NOT_GIVEN,
        preferred_vendor_id: str | NotGiven = NOT_GIVEN,
        purchase_cost: str | NotGiven = NOT_GIVEN,
        purchase_description: str | NotGiven = NOT_GIVEN,
        purchase_tax_code_id: str | NotGiven = NOT_GIVEN,
        quantity_on_hand: float | NotGiven = NOT_GIVEN,
        sales_description: str | NotGiven = NOT_GIVEN,
        sales_price: str | NotGiven = NOT_GIVEN,
        sales_tax_code_id: str | NotGiven = NOT_GIVEN,
        total_value: str | NotGiven = NOT_GIVEN,
        unit_of_measure_set_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InventoryAssemblyItem:
        """
        Creates a new inventory assembly item.

        Args:
          asset_account_id: The asset account used to track the current value of this inventory assembly
              item in inventory.

          cogs_account_id: The Cost of Goods Sold (COGS) account for this inventory assembly item, tracking
              the original direct costs of producing goods sold.

          income_account_id: The income account used to track revenue from sales of this inventory assembly
              item.

          name: The case-insensitive name of this inventory assembly item. Not guaranteed to be
              unique because it does not include the names of its hierarchical parent objects
              like `fullName` does. For example, two inventory assembly items could both have
              the `name` "Deluxe Kit", but they could have unique `fullName` values, such as
              "Assemblies:Deluxe Kit" and "Inventory:Deluxe Kit".

              Maximum length: 31 characters.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          barcode: The inventory assembly item's barcode.

          build_notification_threshold: The inventory assembly item's minimum quantity threshold that triggers a build
              notification in QuickBooks. When the sum of `quantityOnHand` (current inventory)
              and `quantityOnOrder` (pending purchase orders) drops below this threshold,
              QuickBooks will notify users that more units need to be built or assembled. This
              helps ensure adequate inventory levels for inventory assembly items.

          class_id: The inventory assembly item's class. Classes can be used to categorize objects
              into meaningful segments, such as department, location, or type of work. In
              QuickBooks, class tracking is off by default.

          external_id: A globally unique identifier (GUID) you, the developer, can provide for tracking
              this object in your external system. This field is immutable and can only be set
              during object creation.

              **IMPORTANT**: This field must be formatted as a valid GUID; otherwise,
              QuickBooks will return an error.

          inventory_date: The date when this inventory assembly item was converted into an inventory item
              from some other type of item, in ISO 8601 format (YYYY-MM-DD).

          is_active: Indicates whether this inventory assembly item is active. Inactive objects are
              typically hidden from views and reports in QuickBooks. Defaults to `true`.

          lines: The inventory assembly item's lines.

          maximum_quantity_on_hand: The maximum quantity of this inventory assembly item desired in inventory.

          parent_id: The parent inventory assembly item one level above this one in the hierarchy.
              For example, if this inventory assembly item has a `fullName` of
              "Assemblies:Deluxe Kit", its parent has a `fullName` of "Assemblies". If this
              inventory assembly item is at the top level, this field will be `null`.

          preferred_vendor_id: The preferred vendor from whom this inventory assembly item is typically
              purchased.

          purchase_cost: The cost at which this inventory assembly item is purchased from vendors,
              represented as a decimal string.

          purchase_description: The description of this inventory assembly item that appears on purchase forms
              (e.g., checks, bills, item receipts) when it is ordered or bought from vendors.

          purchase_tax_code_id: The tax code applied to purchases of this inventory assembly item. Applicable in
              regions where purchase taxes are used, such as Canada or the UK.

          quantity_on_hand: The current quantity of this inventory assembly item available in inventory. To
              change the `quantityOnHand` for an inventory assembly item, you must create an
              inventory-adjustment instead of updating this inventory assembly item directly.

          sales_description: The description of this inventory assembly item that appears on sales forms
              (e.g., invoices, sales receipts) when sold to customers.

          sales_price: The price at which this inventory assembly item is sold to customers,
              represented as a decimal string.

          sales_tax_code_id: The default sales-tax code for this inventory assembly item, determining whether
              it is taxable or non-taxable. This can be overridden at the transaction-line
              level.

              Default codes include "Non" (non-taxable) and "Tax" (taxable), but custom codes
              can also be created in QuickBooks. If QuickBooks is not set up to charge sales
              tax (via the "Do You Charge Sales Tax?" preference), it will assign the default
              non-taxable code to all sales.

          total_value: The total value of this inventory assembly item. If `totalValue` is provided,
              `quantityOnHand` must also be provided and must be greater than zero. If both
              `quantityOnHand` and `purchaseCost` are provided, then `totalValue` will be set
              to `quantityOnHand` times `purchaseCost`, regardless of what `totalValue` is
              explicitly set to.

          unit_of_measure_set_id: The unit-of-measure set associated with this inventory assembly item, which
              consists of a base unit and related units.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._post(
            "/quickbooks-desktop/inventory-assembly-items",
            body=maybe_transform(
                {
                    "asset_account_id": asset_account_id,
                    "cogs_account_id": cogs_account_id,
                    "income_account_id": income_account_id,
                    "name": name,
                    "barcode": barcode,
                    "build_notification_threshold": build_notification_threshold,
                    "class_id": class_id,
                    "external_id": external_id,
                    "inventory_date": inventory_date,
                    "is_active": is_active,
                    "lines": lines,
                    "maximum_quantity_on_hand": maximum_quantity_on_hand,
                    "parent_id": parent_id,
                    "preferred_vendor_id": preferred_vendor_id,
                    "purchase_cost": purchase_cost,
                    "purchase_description": purchase_description,
                    "purchase_tax_code_id": purchase_tax_code_id,
                    "quantity_on_hand": quantity_on_hand,
                    "sales_description": sales_description,
                    "sales_price": sales_price,
                    "sales_tax_code_id": sales_tax_code_id,
                    "total_value": total_value,
                    "unit_of_measure_set_id": unit_of_measure_set_id,
                },
                inventory_assembly_item_create_params.InventoryAssemblyItemCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InventoryAssemblyItem,
        )

    def retrieve(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InventoryAssemblyItem:
        """
        Retrieves an inventory assembly item by ID.

        Args:
          id: The QuickBooks-assigned unique identifier of the inventory assembly item to
              retrieve.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get(
            f"/quickbooks-desktop/inventory-assembly-items/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InventoryAssemblyItem,
        )

    def update(
        self,
        id: str,
        *,
        revision_number: str,
        conductor_end_user_id: str,
        asset_account_id: str | NotGiven = NOT_GIVEN,
        barcode: inventory_assembly_item_update_params.Barcode | NotGiven = NOT_GIVEN,
        build_notification_threshold: float | NotGiven = NOT_GIVEN,
        class_id: str | NotGiven = NOT_GIVEN,
        clear_item_lines: bool | NotGiven = NOT_GIVEN,
        cogs_account_id: str | NotGiven = NOT_GIVEN,
        force_unit_of_measure_change: bool | NotGiven = NOT_GIVEN,
        income_account_id: str | NotGiven = NOT_GIVEN,
        is_active: bool | NotGiven = NOT_GIVEN,
        lines: Iterable[inventory_assembly_item_update_params.Line] | NotGiven = NOT_GIVEN,
        maximum_quantity_on_hand: float | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        parent_id: str | NotGiven = NOT_GIVEN,
        preferred_vendor_id: str | NotGiven = NOT_GIVEN,
        purchase_cost: str | NotGiven = NOT_GIVEN,
        purchase_description: str | NotGiven = NOT_GIVEN,
        purchase_tax_code_id: str | NotGiven = NOT_GIVEN,
        sales_description: str | NotGiven = NOT_GIVEN,
        sales_price: str | NotGiven = NOT_GIVEN,
        sales_tax_code_id: str | NotGiven = NOT_GIVEN,
        sku: str | NotGiven = NOT_GIVEN,
        unit_of_measure_set_id: str | NotGiven = NOT_GIVEN,
        update_existing_transactions_income_account: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InventoryAssemblyItem:
        """
        Updates an existing inventory assembly item.

        Args:
          id: The QuickBooks-assigned unique identifier of the inventory assembly item to
              update.

          revision_number: The current QuickBooks-assigned revision number of the inventory assembly item
              object you are updating, which you can get by fetching the object first. Provide
              the most recent `revisionNumber` to ensure you're working with the latest data;
              otherwise, the update will return an error.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          asset_account_id: The asset account used to track the current value of this inventory assembly
              item in inventory.

          barcode: The inventory assembly item's barcode.

          build_notification_threshold: The inventory assembly item's minimum quantity threshold that triggers a build
              notification in QuickBooks. When the sum of `quantityOnHand` (current inventory)
              and `quantityOnOrder` (pending purchase orders) drops below this threshold,
              QuickBooks will notify users that more units need to be built or assembled. This
              helps ensure adequate inventory levels for inventory assembly items.

          class_id: The inventory assembly item's class. Classes can be used to categorize objects
              into meaningful segments, such as department, location, or type of work. In
              QuickBooks, class tracking is off by default.

          clear_item_lines: When `true`, removes all existing item lines associated with this inventory
              assembly item. To modify or add individual item lines, use the field `itemLines`
              instead.

          cogs_account_id: The Cost of Goods Sold (COGS) account for this inventory assembly item, tracking
              the original direct costs of producing goods sold.

          force_unit_of_measure_change: Indicates whether to allow changing the inventory assembly item's
              unit-of-measure set (using the `unitOfMeasureSetId` field) when the base unit of
              the new unit-of-measure set does not match that of the currently assigned set.
              Without setting this field to `true` in this scenario, the request will fail
              with an error; hence, this field is equivalent to accepting the warning prompt
              in the QuickBooks UI.

              NOTE: Changing the base unit requires you to update the item's
              quantities-on-hand and cost to reflect the new unit; otherwise, these values
              will be inaccurate. Alternatively, consider creating a new item with the desired
              unit-of-measure set and deactivating the old item.

          income_account_id: The income account used to track revenue from sales of this inventory assembly
              item.

          is_active: Indicates whether this inventory assembly item is active. Inactive objects are
              typically hidden from views and reports in QuickBooks. Defaults to `true`.

          lines: The inventory assembly item's lines.

          maximum_quantity_on_hand: The maximum quantity of this inventory assembly item desired in inventory.

          name: The case-insensitive name of this inventory assembly item. Not guaranteed to be
              unique because it does not include the names of its hierarchical parent objects
              like `fullName` does. For example, two inventory assembly items could both have
              the `name` "Deluxe Kit", but they could have unique `fullName` values, such as
              "Assemblies:Deluxe Kit" and "Inventory:Deluxe Kit".

              Maximum length: 31 characters.

          parent_id: The parent inventory assembly item one level above this one in the hierarchy.
              For example, if this inventory assembly item has a `fullName` of
              "Assemblies:Deluxe Kit", its parent has a `fullName` of "Assemblies". If this
              inventory assembly item is at the top level, this field will be `null`.

          preferred_vendor_id: The preferred vendor from whom this inventory assembly item is typically
              purchased.

          purchase_cost: The cost at which this inventory assembly item is purchased from vendors,
              represented as a decimal string.

          purchase_description: The description of this inventory assembly item that appears on purchase forms
              (e.g., checks, bills, item receipts) when it is ordered or bought from vendors.

          purchase_tax_code_id: The tax code applied to purchases of this inventory assembly item. Applicable in
              regions where purchase taxes are used, such as Canada or the UK.

          sales_description: The description of this inventory assembly item that appears on sales forms
              (e.g., invoices, sales receipts) when sold to customers.

          sales_price: The price at which this inventory assembly item is sold to customers,
              represented as a decimal string.

          sales_tax_code_id: The default sales-tax code for this inventory assembly item, determining whether
              it is taxable or non-taxable. This can be overridden at the transaction-line
              level.

              Default codes include "Non" (non-taxable) and "Tax" (taxable), but custom codes
              can also be created in QuickBooks. If QuickBooks is not set up to charge sales
              tax (via the "Do You Charge Sales Tax?" preference), it will assign the default
              non-taxable code to all sales.

          sku: The inventory assembly item's stock keeping unit (SKU), which is sometimes the
              manufacturer's part number.

          unit_of_measure_set_id: The unit-of-measure set associated with this inventory assembly item, which
              consists of a base unit and related units.

          update_existing_transactions_income_account: When `true`, applies the new income account (specified by the `incomeAccountId`
              field) to all existing transactions that use this inventory assembly item. This
              updates historical data and should be used with caution. The update will fail if
              any affected transaction falls within a closed accounting period. If this
              parameter is not specified, QuickBooks will prompt the user before making any
              changes.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._post(
            f"/quickbooks-desktop/inventory-assembly-items/{id}",
            body=maybe_transform(
                {
                    "revision_number": revision_number,
                    "asset_account_id": asset_account_id,
                    "barcode": barcode,
                    "build_notification_threshold": build_notification_threshold,
                    "class_id": class_id,
                    "clear_item_lines": clear_item_lines,
                    "cogs_account_id": cogs_account_id,
                    "force_unit_of_measure_change": force_unit_of_measure_change,
                    "income_account_id": income_account_id,
                    "is_active": is_active,
                    "lines": lines,
                    "maximum_quantity_on_hand": maximum_quantity_on_hand,
                    "name": name,
                    "parent_id": parent_id,
                    "preferred_vendor_id": preferred_vendor_id,
                    "purchase_cost": purchase_cost,
                    "purchase_description": purchase_description,
                    "purchase_tax_code_id": purchase_tax_code_id,
                    "sales_description": sales_description,
                    "sales_price": sales_price,
                    "sales_tax_code_id": sales_tax_code_id,
                    "sku": sku,
                    "unit_of_measure_set_id": unit_of_measure_set_id,
                    "update_existing_transactions_income_account": update_existing_transactions_income_account,
                },
                inventory_assembly_item_update_params.InventoryAssemblyItemUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InventoryAssemblyItem,
        )

    def list(
        self,
        *,
        conductor_end_user_id: str,
        class_ids: List[str] | NotGiven = NOT_GIVEN,
        cursor: str | NotGiven = NOT_GIVEN,
        full_names: List[str] | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        name_contains: str | NotGiven = NOT_GIVEN,
        name_ends_with: str | NotGiven = NOT_GIVEN,
        name_from: str | NotGiven = NOT_GIVEN,
        name_starts_with: str | NotGiven = NOT_GIVEN,
        name_to: str | NotGiven = NOT_GIVEN,
        status: Literal["active", "all", "inactive"] | NotGiven = NOT_GIVEN,
        updated_after: str | NotGiven = NOT_GIVEN,
        updated_before: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SyncCursorPage[InventoryAssemblyItem]:
        """Returns a list of inventory assembly items.

        Use the `cursor` parameter to
        paginate through the results.

        Args:
          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          class_ids: Filter for inventory assembly items of these classes. A class is a way end-users
              can categorize inventory assembly items in QuickBooks.

          cursor: The pagination token to fetch the next set of results when paginating with the
              `limit` parameter. Do not include this parameter on the first call. Use the
              `nextCursor` value returned in the previous response to request subsequent
              results.

          full_names: Filter for specific inventory assembly items by their full-name(s),
              case-insensitive. Like `id`, `fullName` is a unique identifier for an inventory
              assembly item, formed by by combining the names of its parent objects with its
              own `name`, separated by colons. For example, if an inventory assembly item is
              under "Assemblies" and has the `name` "Deluxe Kit", its `fullName` would be
              "Assemblies:Deluxe Kit".

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          ids: Filter for specific inventory assembly items by their QuickBooks-assigned unique
              identifier(s).

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          limit: The maximum number of objects to return. Accepts values ranging from 1 to 150,
              defaults to 150. When used with cursor-based pagination, this parameter controls
              how many results are returned per page. To paginate through results, combine
              this with the `cursor` parameter. Each response will include a `nextCursor`
              value that can be passed to subsequent requests to retrieve the next page of
              results.

          name_contains: Filter for inventory assembly items whose `name` contains this substring,
              case-insensitive. NOTE: If you use this parameter, you cannot also use
              `nameStartsWith` or `nameEndsWith`.

          name_ends_with: Filter for inventory assembly items whose `name` ends with this substring,
              case-insensitive. NOTE: If you use this parameter, you cannot also use
              `nameContains` or `nameStartsWith`.

          name_from: Filter for inventory assembly items whose `name` is alphabetically greater than
              or equal to this value.

          name_starts_with: Filter for inventory assembly items whose `name` starts with this substring,
              case-insensitive. NOTE: If you use this parameter, you cannot also use
              `nameContains` or `nameEndsWith`.

          name_to: Filter for inventory assembly items whose `name` is alphabetically less than or
              equal to this value.

          status: Filter for inventory assembly items that are active, inactive, or both.

          updated_after: Filter for inventory assembly items updated on or after this date and time, in
              ISO 8601 format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD),
              the time is assumed to be 00:00:00 of that day.

              **WARNING**: Due to a known issue in QuickBooks Desktop, the `updatedAfter`
              parameter may not correctly filter inventory assembly items by their updated
              dates. To accurately retrieve the desired inventory assembly items, we recommend
              avoiding this parameter and instead fetching a broader dataset, then filtering
              the results locally using the `updatedAt` property.

          updated_before: Filter for inventory assembly items updated on or before this date and time, in
              ISO 8601 format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD),
              the time is assumed to be 23:59:59 of that day.

              **WARNING**: Due to a known issue in QuickBooks Desktop, the `updatedBefore`
              parameter may not correctly filter inventory assembly items by their updated
              dates. To accurately retrieve the desired inventory assembly items, we recommend
              avoiding this parameter and instead fetching a broader dataset, then filtering
              the results locally using the `updatedAt` property.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get_api_list(
            "/quickbooks-desktop/inventory-assembly-items",
            page=SyncCursorPage[InventoryAssemblyItem],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "class_ids": class_ids,
                        "cursor": cursor,
                        "full_names": full_names,
                        "ids": ids,
                        "limit": limit,
                        "name_contains": name_contains,
                        "name_ends_with": name_ends_with,
                        "name_from": name_from,
                        "name_starts_with": name_starts_with,
                        "name_to": name_to,
                        "status": status,
                        "updated_after": updated_after,
                        "updated_before": updated_before,
                    },
                    inventory_assembly_item_list_params.InventoryAssemblyItemListParams,
                ),
            ),
            model=InventoryAssemblyItem,
        )


class AsyncInventoryAssemblyItemsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncInventoryAssemblyItemsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#accessing-raw-response-data-eg-headers
        """
        return AsyncInventoryAssemblyItemsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInventoryAssemblyItemsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/conductor-is/quickbooks-desktop-python#with_streaming_response
        """
        return AsyncInventoryAssemblyItemsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        asset_account_id: str,
        cogs_account_id: str,
        income_account_id: str,
        name: str,
        conductor_end_user_id: str,
        barcode: inventory_assembly_item_create_params.Barcode | NotGiven = NOT_GIVEN,
        build_notification_threshold: float | NotGiven = NOT_GIVEN,
        class_id: str | NotGiven = NOT_GIVEN,
        external_id: str | NotGiven = NOT_GIVEN,
        inventory_date: Union[str, date] | NotGiven = NOT_GIVEN,
        is_active: bool | NotGiven = NOT_GIVEN,
        lines: Iterable[inventory_assembly_item_create_params.Line] | NotGiven = NOT_GIVEN,
        maximum_quantity_on_hand: float | NotGiven = NOT_GIVEN,
        parent_id: str | NotGiven = NOT_GIVEN,
        preferred_vendor_id: str | NotGiven = NOT_GIVEN,
        purchase_cost: str | NotGiven = NOT_GIVEN,
        purchase_description: str | NotGiven = NOT_GIVEN,
        purchase_tax_code_id: str | NotGiven = NOT_GIVEN,
        quantity_on_hand: float | NotGiven = NOT_GIVEN,
        sales_description: str | NotGiven = NOT_GIVEN,
        sales_price: str | NotGiven = NOT_GIVEN,
        sales_tax_code_id: str | NotGiven = NOT_GIVEN,
        total_value: str | NotGiven = NOT_GIVEN,
        unit_of_measure_set_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InventoryAssemblyItem:
        """
        Creates a new inventory assembly item.

        Args:
          asset_account_id: The asset account used to track the current value of this inventory assembly
              item in inventory.

          cogs_account_id: The Cost of Goods Sold (COGS) account for this inventory assembly item, tracking
              the original direct costs of producing goods sold.

          income_account_id: The income account used to track revenue from sales of this inventory assembly
              item.

          name: The case-insensitive name of this inventory assembly item. Not guaranteed to be
              unique because it does not include the names of its hierarchical parent objects
              like `fullName` does. For example, two inventory assembly items could both have
              the `name` "Deluxe Kit", but they could have unique `fullName` values, such as
              "Assemblies:Deluxe Kit" and "Inventory:Deluxe Kit".

              Maximum length: 31 characters.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          barcode: The inventory assembly item's barcode.

          build_notification_threshold: The inventory assembly item's minimum quantity threshold that triggers a build
              notification in QuickBooks. When the sum of `quantityOnHand` (current inventory)
              and `quantityOnOrder` (pending purchase orders) drops below this threshold,
              QuickBooks will notify users that more units need to be built or assembled. This
              helps ensure adequate inventory levels for inventory assembly items.

          class_id: The inventory assembly item's class. Classes can be used to categorize objects
              into meaningful segments, such as department, location, or type of work. In
              QuickBooks, class tracking is off by default.

          external_id: A globally unique identifier (GUID) you, the developer, can provide for tracking
              this object in your external system. This field is immutable and can only be set
              during object creation.

              **IMPORTANT**: This field must be formatted as a valid GUID; otherwise,
              QuickBooks will return an error.

          inventory_date: The date when this inventory assembly item was converted into an inventory item
              from some other type of item, in ISO 8601 format (YYYY-MM-DD).

          is_active: Indicates whether this inventory assembly item is active. Inactive objects are
              typically hidden from views and reports in QuickBooks. Defaults to `true`.

          lines: The inventory assembly item's lines.

          maximum_quantity_on_hand: The maximum quantity of this inventory assembly item desired in inventory.

          parent_id: The parent inventory assembly item one level above this one in the hierarchy.
              For example, if this inventory assembly item has a `fullName` of
              "Assemblies:Deluxe Kit", its parent has a `fullName` of "Assemblies". If this
              inventory assembly item is at the top level, this field will be `null`.

          preferred_vendor_id: The preferred vendor from whom this inventory assembly item is typically
              purchased.

          purchase_cost: The cost at which this inventory assembly item is purchased from vendors,
              represented as a decimal string.

          purchase_description: The description of this inventory assembly item that appears on purchase forms
              (e.g., checks, bills, item receipts) when it is ordered or bought from vendors.

          purchase_tax_code_id: The tax code applied to purchases of this inventory assembly item. Applicable in
              regions where purchase taxes are used, such as Canada or the UK.

          quantity_on_hand: The current quantity of this inventory assembly item available in inventory. To
              change the `quantityOnHand` for an inventory assembly item, you must create an
              inventory-adjustment instead of updating this inventory assembly item directly.

          sales_description: The description of this inventory assembly item that appears on sales forms
              (e.g., invoices, sales receipts) when sold to customers.

          sales_price: The price at which this inventory assembly item is sold to customers,
              represented as a decimal string.

          sales_tax_code_id: The default sales-tax code for this inventory assembly item, determining whether
              it is taxable or non-taxable. This can be overridden at the transaction-line
              level.

              Default codes include "Non" (non-taxable) and "Tax" (taxable), but custom codes
              can also be created in QuickBooks. If QuickBooks is not set up to charge sales
              tax (via the "Do You Charge Sales Tax?" preference), it will assign the default
              non-taxable code to all sales.

          total_value: The total value of this inventory assembly item. If `totalValue` is provided,
              `quantityOnHand` must also be provided and must be greater than zero. If both
              `quantityOnHand` and `purchaseCost` are provided, then `totalValue` will be set
              to `quantityOnHand` times `purchaseCost`, regardless of what `totalValue` is
              explicitly set to.

          unit_of_measure_set_id: The unit-of-measure set associated with this inventory assembly item, which
              consists of a base unit and related units.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._post(
            "/quickbooks-desktop/inventory-assembly-items",
            body=await async_maybe_transform(
                {
                    "asset_account_id": asset_account_id,
                    "cogs_account_id": cogs_account_id,
                    "income_account_id": income_account_id,
                    "name": name,
                    "barcode": barcode,
                    "build_notification_threshold": build_notification_threshold,
                    "class_id": class_id,
                    "external_id": external_id,
                    "inventory_date": inventory_date,
                    "is_active": is_active,
                    "lines": lines,
                    "maximum_quantity_on_hand": maximum_quantity_on_hand,
                    "parent_id": parent_id,
                    "preferred_vendor_id": preferred_vendor_id,
                    "purchase_cost": purchase_cost,
                    "purchase_description": purchase_description,
                    "purchase_tax_code_id": purchase_tax_code_id,
                    "quantity_on_hand": quantity_on_hand,
                    "sales_description": sales_description,
                    "sales_price": sales_price,
                    "sales_tax_code_id": sales_tax_code_id,
                    "total_value": total_value,
                    "unit_of_measure_set_id": unit_of_measure_set_id,
                },
                inventory_assembly_item_create_params.InventoryAssemblyItemCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InventoryAssemblyItem,
        )

    async def retrieve(
        self,
        id: str,
        *,
        conductor_end_user_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InventoryAssemblyItem:
        """
        Retrieves an inventory assembly item by ID.

        Args:
          id: The QuickBooks-assigned unique identifier of the inventory assembly item to
              retrieve.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._get(
            f"/quickbooks-desktop/inventory-assembly-items/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InventoryAssemblyItem,
        )

    async def update(
        self,
        id: str,
        *,
        revision_number: str,
        conductor_end_user_id: str,
        asset_account_id: str | NotGiven = NOT_GIVEN,
        barcode: inventory_assembly_item_update_params.Barcode | NotGiven = NOT_GIVEN,
        build_notification_threshold: float | NotGiven = NOT_GIVEN,
        class_id: str | NotGiven = NOT_GIVEN,
        clear_item_lines: bool | NotGiven = NOT_GIVEN,
        cogs_account_id: str | NotGiven = NOT_GIVEN,
        force_unit_of_measure_change: bool | NotGiven = NOT_GIVEN,
        income_account_id: str | NotGiven = NOT_GIVEN,
        is_active: bool | NotGiven = NOT_GIVEN,
        lines: Iterable[inventory_assembly_item_update_params.Line] | NotGiven = NOT_GIVEN,
        maximum_quantity_on_hand: float | NotGiven = NOT_GIVEN,
        name: str | NotGiven = NOT_GIVEN,
        parent_id: str | NotGiven = NOT_GIVEN,
        preferred_vendor_id: str | NotGiven = NOT_GIVEN,
        purchase_cost: str | NotGiven = NOT_GIVEN,
        purchase_description: str | NotGiven = NOT_GIVEN,
        purchase_tax_code_id: str | NotGiven = NOT_GIVEN,
        sales_description: str | NotGiven = NOT_GIVEN,
        sales_price: str | NotGiven = NOT_GIVEN,
        sales_tax_code_id: str | NotGiven = NOT_GIVEN,
        sku: str | NotGiven = NOT_GIVEN,
        unit_of_measure_set_id: str | NotGiven = NOT_GIVEN,
        update_existing_transactions_income_account: bool | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> InventoryAssemblyItem:
        """
        Updates an existing inventory assembly item.

        Args:
          id: The QuickBooks-assigned unique identifier of the inventory assembly item to
              update.

          revision_number: The current QuickBooks-assigned revision number of the inventory assembly item
              object you are updating, which you can get by fetching the object first. Provide
              the most recent `revisionNumber` to ensure you're working with the latest data;
              otherwise, the update will return an error.

          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          asset_account_id: The asset account used to track the current value of this inventory assembly
              item in inventory.

          barcode: The inventory assembly item's barcode.

          build_notification_threshold: The inventory assembly item's minimum quantity threshold that triggers a build
              notification in QuickBooks. When the sum of `quantityOnHand` (current inventory)
              and `quantityOnOrder` (pending purchase orders) drops below this threshold,
              QuickBooks will notify users that more units need to be built or assembled. This
              helps ensure adequate inventory levels for inventory assembly items.

          class_id: The inventory assembly item's class. Classes can be used to categorize objects
              into meaningful segments, such as department, location, or type of work. In
              QuickBooks, class tracking is off by default.

          clear_item_lines: When `true`, removes all existing item lines associated with this inventory
              assembly item. To modify or add individual item lines, use the field `itemLines`
              instead.

          cogs_account_id: The Cost of Goods Sold (COGS) account for this inventory assembly item, tracking
              the original direct costs of producing goods sold.

          force_unit_of_measure_change: Indicates whether to allow changing the inventory assembly item's
              unit-of-measure set (using the `unitOfMeasureSetId` field) when the base unit of
              the new unit-of-measure set does not match that of the currently assigned set.
              Without setting this field to `true` in this scenario, the request will fail
              with an error; hence, this field is equivalent to accepting the warning prompt
              in the QuickBooks UI.

              NOTE: Changing the base unit requires you to update the item's
              quantities-on-hand and cost to reflect the new unit; otherwise, these values
              will be inaccurate. Alternatively, consider creating a new item with the desired
              unit-of-measure set and deactivating the old item.

          income_account_id: The income account used to track revenue from sales of this inventory assembly
              item.

          is_active: Indicates whether this inventory assembly item is active. Inactive objects are
              typically hidden from views and reports in QuickBooks. Defaults to `true`.

          lines: The inventory assembly item's lines.

          maximum_quantity_on_hand: The maximum quantity of this inventory assembly item desired in inventory.

          name: The case-insensitive name of this inventory assembly item. Not guaranteed to be
              unique because it does not include the names of its hierarchical parent objects
              like `fullName` does. For example, two inventory assembly items could both have
              the `name` "Deluxe Kit", but they could have unique `fullName` values, such as
              "Assemblies:Deluxe Kit" and "Inventory:Deluxe Kit".

              Maximum length: 31 characters.

          parent_id: The parent inventory assembly item one level above this one in the hierarchy.
              For example, if this inventory assembly item has a `fullName` of
              "Assemblies:Deluxe Kit", its parent has a `fullName` of "Assemblies". If this
              inventory assembly item is at the top level, this field will be `null`.

          preferred_vendor_id: The preferred vendor from whom this inventory assembly item is typically
              purchased.

          purchase_cost: The cost at which this inventory assembly item is purchased from vendors,
              represented as a decimal string.

          purchase_description: The description of this inventory assembly item that appears on purchase forms
              (e.g., checks, bills, item receipts) when it is ordered or bought from vendors.

          purchase_tax_code_id: The tax code applied to purchases of this inventory assembly item. Applicable in
              regions where purchase taxes are used, such as Canada or the UK.

          sales_description: The description of this inventory assembly item that appears on sales forms
              (e.g., invoices, sales receipts) when sold to customers.

          sales_price: The price at which this inventory assembly item is sold to customers,
              represented as a decimal string.

          sales_tax_code_id: The default sales-tax code for this inventory assembly item, determining whether
              it is taxable or non-taxable. This can be overridden at the transaction-line
              level.

              Default codes include "Non" (non-taxable) and "Tax" (taxable), but custom codes
              can also be created in QuickBooks. If QuickBooks is not set up to charge sales
              tax (via the "Do You Charge Sales Tax?" preference), it will assign the default
              non-taxable code to all sales.

          sku: The inventory assembly item's stock keeping unit (SKU), which is sometimes the
              manufacturer's part number.

          unit_of_measure_set_id: The unit-of-measure set associated with this inventory assembly item, which
              consists of a base unit and related units.

          update_existing_transactions_income_account: When `true`, applies the new income account (specified by the `incomeAccountId`
              field) to all existing transactions that use this inventory assembly item. This
              updates historical data and should be used with caution. The update will fail if
              any affected transaction falls within a closed accounting period. If this
              parameter is not specified, QuickBooks will prompt the user before making any
              changes.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return await self._post(
            f"/quickbooks-desktop/inventory-assembly-items/{id}",
            body=await async_maybe_transform(
                {
                    "revision_number": revision_number,
                    "asset_account_id": asset_account_id,
                    "barcode": barcode,
                    "build_notification_threshold": build_notification_threshold,
                    "class_id": class_id,
                    "clear_item_lines": clear_item_lines,
                    "cogs_account_id": cogs_account_id,
                    "force_unit_of_measure_change": force_unit_of_measure_change,
                    "income_account_id": income_account_id,
                    "is_active": is_active,
                    "lines": lines,
                    "maximum_quantity_on_hand": maximum_quantity_on_hand,
                    "name": name,
                    "parent_id": parent_id,
                    "preferred_vendor_id": preferred_vendor_id,
                    "purchase_cost": purchase_cost,
                    "purchase_description": purchase_description,
                    "purchase_tax_code_id": purchase_tax_code_id,
                    "sales_description": sales_description,
                    "sales_price": sales_price,
                    "sales_tax_code_id": sales_tax_code_id,
                    "sku": sku,
                    "unit_of_measure_set_id": unit_of_measure_set_id,
                    "update_existing_transactions_income_account": update_existing_transactions_income_account,
                },
                inventory_assembly_item_update_params.InventoryAssemblyItemUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=InventoryAssemblyItem,
        )

    def list(
        self,
        *,
        conductor_end_user_id: str,
        class_ids: List[str] | NotGiven = NOT_GIVEN,
        cursor: str | NotGiven = NOT_GIVEN,
        full_names: List[str] | NotGiven = NOT_GIVEN,
        ids: List[str] | NotGiven = NOT_GIVEN,
        limit: int | NotGiven = NOT_GIVEN,
        name_contains: str | NotGiven = NOT_GIVEN,
        name_ends_with: str | NotGiven = NOT_GIVEN,
        name_from: str | NotGiven = NOT_GIVEN,
        name_starts_with: str | NotGiven = NOT_GIVEN,
        name_to: str | NotGiven = NOT_GIVEN,
        status: Literal["active", "all", "inactive"] | NotGiven = NOT_GIVEN,
        updated_after: str | NotGiven = NOT_GIVEN,
        updated_before: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> AsyncPaginator[InventoryAssemblyItem, AsyncCursorPage[InventoryAssemblyItem]]:
        """Returns a list of inventory assembly items.

        Use the `cursor` parameter to
        paginate through the results.

        Args:
          conductor_end_user_id: The ID of the EndUser to receive this request (e.g.,
              `"Conductor-End-User-Id: {{END_USER_ID}}"`).

          class_ids: Filter for inventory assembly items of these classes. A class is a way end-users
              can categorize inventory assembly items in QuickBooks.

          cursor: The pagination token to fetch the next set of results when paginating with the
              `limit` parameter. Do not include this parameter on the first call. Use the
              `nextCursor` value returned in the previous response to request subsequent
              results.

          full_names: Filter for specific inventory assembly items by their full-name(s),
              case-insensitive. Like `id`, `fullName` is a unique identifier for an inventory
              assembly item, formed by by combining the names of its parent objects with its
              own `name`, separated by colons. For example, if an inventory assembly item is
              under "Assemblies" and has the `name` "Deluxe Kit", its `fullName` would be
              "Assemblies:Deluxe Kit".

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          ids: Filter for specific inventory assembly items by their QuickBooks-assigned unique
              identifier(s).

              **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
              query parameters for this request.

              **NOTE**: If any of the values you specify in this parameter are not found, the
              request will return an error.

          limit: The maximum number of objects to return. Accepts values ranging from 1 to 150,
              defaults to 150. When used with cursor-based pagination, this parameter controls
              how many results are returned per page. To paginate through results, combine
              this with the `cursor` parameter. Each response will include a `nextCursor`
              value that can be passed to subsequent requests to retrieve the next page of
              results.

          name_contains: Filter for inventory assembly items whose `name` contains this substring,
              case-insensitive. NOTE: If you use this parameter, you cannot also use
              `nameStartsWith` or `nameEndsWith`.

          name_ends_with: Filter for inventory assembly items whose `name` ends with this substring,
              case-insensitive. NOTE: If you use this parameter, you cannot also use
              `nameContains` or `nameStartsWith`.

          name_from: Filter for inventory assembly items whose `name` is alphabetically greater than
              or equal to this value.

          name_starts_with: Filter for inventory assembly items whose `name` starts with this substring,
              case-insensitive. NOTE: If you use this parameter, you cannot also use
              `nameContains` or `nameEndsWith`.

          name_to: Filter for inventory assembly items whose `name` is alphabetically less than or
              equal to this value.

          status: Filter for inventory assembly items that are active, inactive, or both.

          updated_after: Filter for inventory assembly items updated on or after this date and time, in
              ISO 8601 format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD),
              the time is assumed to be 00:00:00 of that day.

              **WARNING**: Due to a known issue in QuickBooks Desktop, the `updatedAfter`
              parameter may not correctly filter inventory assembly items by their updated
              dates. To accurately retrieve the desired inventory assembly items, we recommend
              avoiding this parameter and instead fetching a broader dataset, then filtering
              the results locally using the `updatedAt` property.

          updated_before: Filter for inventory assembly items updated on or before this date and time, in
              ISO 8601 format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD),
              the time is assumed to be 23:59:59 of that day.

              **WARNING**: Due to a known issue in QuickBooks Desktop, the `updatedBefore`
              parameter may not correctly filter inventory assembly items by their updated
              dates. To accurately retrieve the desired inventory assembly items, we recommend
              avoiding this parameter and instead fetching a broader dataset, then filtering
              the results locally using the `updatedAt` property.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Conductor-End-User-Id": conductor_end_user_id, **(extra_headers or {})}
        return self._get_api_list(
            "/quickbooks-desktop/inventory-assembly-items",
            page=AsyncCursorPage[InventoryAssemblyItem],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "class_ids": class_ids,
                        "cursor": cursor,
                        "full_names": full_names,
                        "ids": ids,
                        "limit": limit,
                        "name_contains": name_contains,
                        "name_ends_with": name_ends_with,
                        "name_from": name_from,
                        "name_starts_with": name_starts_with,
                        "name_to": name_to,
                        "status": status,
                        "updated_after": updated_after,
                        "updated_before": updated_before,
                    },
                    inventory_assembly_item_list_params.InventoryAssemblyItemListParams,
                ),
            ),
            model=InventoryAssemblyItem,
        )


class InventoryAssemblyItemsResourceWithRawResponse:
    def __init__(self, inventory_assembly_items: InventoryAssemblyItemsResource) -> None:
        self._inventory_assembly_items = inventory_assembly_items

        self.create = to_raw_response_wrapper(
            inventory_assembly_items.create,
        )
        self.retrieve = to_raw_response_wrapper(
            inventory_assembly_items.retrieve,
        )
        self.update = to_raw_response_wrapper(
            inventory_assembly_items.update,
        )
        self.list = to_raw_response_wrapper(
            inventory_assembly_items.list,
        )


class AsyncInventoryAssemblyItemsResourceWithRawResponse:
    def __init__(self, inventory_assembly_items: AsyncInventoryAssemblyItemsResource) -> None:
        self._inventory_assembly_items = inventory_assembly_items

        self.create = async_to_raw_response_wrapper(
            inventory_assembly_items.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            inventory_assembly_items.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            inventory_assembly_items.update,
        )
        self.list = async_to_raw_response_wrapper(
            inventory_assembly_items.list,
        )


class InventoryAssemblyItemsResourceWithStreamingResponse:
    def __init__(self, inventory_assembly_items: InventoryAssemblyItemsResource) -> None:
        self._inventory_assembly_items = inventory_assembly_items

        self.create = to_streamed_response_wrapper(
            inventory_assembly_items.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            inventory_assembly_items.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            inventory_assembly_items.update,
        )
        self.list = to_streamed_response_wrapper(
            inventory_assembly_items.list,
        )


class AsyncInventoryAssemblyItemsResourceWithStreamingResponse:
    def __init__(self, inventory_assembly_items: AsyncInventoryAssemblyItemsResource) -> None:
        self._inventory_assembly_items = inventory_assembly_items

        self.create = async_to_streamed_response_wrapper(
            inventory_assembly_items.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            inventory_assembly_items.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            inventory_assembly_items.update,
        )
        self.list = async_to_streamed_response_wrapper(
            inventory_assembly_items.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["AccountListParams"]


class AccountListParams(TypedDict, total=False):
    conductor_end_user_id: Required[Annotated[str, PropertyInfo(alias="Conductor-End-User-Id")]]
    """
    The ID of the EndUser to receive this request (e.g.,
    `"Conductor-End-User-Id: {{END_USER_ID}}"`).
    """

    account_type: Annotated[
        Literal[
            "accounts_payable",
            "accounts_receivable",
            "bank",
            "cost_of_goods_sold",
            "credit_card",
            "equity",
            "expense",
            "fixed_asset",
            "income",
            "long_term_liability",
            "non_posting",
            "other_asset",
            "other_current_asset",
            "other_current_liability",
            "other_expense",
            "other_income",
        ],
        PropertyInfo(alias="accountType"),
    ]
    """Filter for accounts of this type."""

    currency_ids: Annotated[List[str], PropertyInfo(alias="currencyIds")]
    """Filter for accounts in these currencies."""

    full_names: Annotated[List[str], PropertyInfo(alias="fullNames")]
    """Filter for specific accounts by their full-name(s), case-insensitive.

    Like `id`, `fullName` is a unique identifier for an account, formed by by
    combining the names of its parent objects with its own `name`, separated by
    colons. For example, if an account is under "Corporate" and has the `name`
    "Accounts-Payable", its `fullName` would be "Corporate:Accounts-Payable".

    **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
    query parameters for this request.

    **NOTE**: If any of the values you specify in this parameter are not found, the
    request will return an error.
    """

    ids: List[str]
    """Filter for specific accounts by their QuickBooks-assigned unique identifier(s).

    **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
    query parameters for this request.

    **NOTE**: If any of the values you specify in this parameter are not found, the
    request will return an error.
    """

    limit: int
    """The maximum number of objects to return.

    **IMPORTANT**: QuickBooks Desktop does not support cursor-based pagination for
    accounts. This parameter will limit the response size, but you cannot fetch
    subsequent results using a cursor. For pagination, use the name-range parameters
    instead (e.g., `nameFrom=A&nameTo=B`).

    When this parameter is omitted, the endpoint returns all accounts without limit,
    unlike paginated endpoints which default to 150 records. This is acceptable
    because accounts typically have low record counts.
    """

    name_contains: Annotated[str, PropertyInfo(alias="nameContains")]
    """Filter for accounts whose `name` contains this substring, case-insensitive.

    NOTE: If you use this parameter, you cannot also use `nameStartsWith` or
    `nameEndsWith`.
    """

    name_ends_with: Annotated[str, PropertyInfo(alias="nameEndsWith")]
    """Filter for accounts whose `name` ends with this substring, case-insensitive.

    NOTE: If you use this parameter, you cannot also use `nameContains` or
    `nameStartsWith`.
    """

    name_from: Annotated[str, PropertyInfo(alias="nameFrom")]
    """
    Filter for accounts whose `name` is alphabetically greater than or equal to this
    value.
    """

    name_starts_with: Annotated[str, PropertyInfo(alias="nameStartsWith")]
    """Filter for accounts whose `name` starts with this substring, case-insensitive.

    NOTE: If you use this parameter, you cannot also use `nameContains` or
    `nameEndsWith`.
    """

    name_to: Annotated[str, PropertyInfo(alias="nameTo")]
    """
    Filter for accounts whose `name` is alphabetically less than or equal to this
    value.
    """

    status: Literal["active", "all", "inactive"]
    """Filter for accounts that are active, inactive, or both."""

    updated_after: Annotated[str, PropertyInfo(alias="updatedAfter")]
    """
    Filter for accounts updated on or after this date and time, in ISO 8601 format
    (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time is
    assumed to be 00:00:00 of that day.
    """

    updated_before: Annotated[str, PropertyInfo(alias="updatedBefore")]
    """
    Filter for accounts updated on or before this date and time, in ISO 8601 format
    (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time is
    assumed to be 23:59:59 of that day.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["SalesRepresentativeListParams"]


class SalesRepresentativeListParams(TypedDict, total=False):
    conductor_end_user_id: Required[Annotated[str, PropertyInfo(alias="Conductor-End-User-Id")]]
    """
    The ID of the EndUser to receive this request (e.g.,
    `"Conductor-End-User-Id: {{END_USER_ID}}"`).
    """

    ids: List[str]
    """
    Filter for specific sales representatives by their QuickBooks-assigned unique
    identifier(s).

    **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
    query parameters for this request.

    **NOTE**: If any of the values you specify in this parameter are not found, the
    request will return an error.
    """

    limit: int
    """The maximum number of objects to return.

    **IMPORTANT**: QuickBooks Desktop does not support cursor-based pagination for
    sales representatives. This parameter will limit the response size, but you
    cannot fetch subsequent results using a cursor. For pagination, use the
    name-range parameters instead (e.g., `nameFrom=A&nameTo=B`).

    When this parameter is omitted, the endpoint returns all sales representatives
    without limit, unlike paginated endpoints which default to 150 records. This is
    acceptable because sales representatives typically have low record counts.
    """

    name_contains: Annotated[str, PropertyInfo(alias="nameContains")]
    """
    Filter for sales representatives whose `name` contains this substring,
    case-insensitive. NOTE: If you use this parameter, you cannot also use
    `nameStartsWith` or `nameEndsWith`.
    """

    name_ends_with: Annotated[str, PropertyInfo(alias="nameEndsWith")]
    """
    Filter for sales representatives whose `name` ends with this substring,
    case-insensitive. NOTE: If you use this parameter, you cannot also use
    `nameContains` or `nameStartsWith`.
    """

    name_from: Annotated[str, PropertyInfo(alias="nameFrom")]
    """
    Filter for sales representatives whose `name` is alphabetically greater than or
    equal to this value.
    """

    names: List[str]
    """Filter for specific sales representatives by their name(s), case-insensitive.

    Like `id`, `name` is a unique identifier for a sales representative.

    **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
    query parameters for this request.

    **NOTE**: If any of the values you specify in this parameter are not found, the
    request will return an error.
    """

    name_starts_with: Annotated[str, PropertyInfo(alias="nameStartsWith")]
    """
    Filter for sales representatives whose `name` starts with this substring,
    case-insensitive. NOTE: If you use this parameter, you cannot also use
    `nameContains` or `nameEndsWith`.
    """

    name_to: Annotated[str, PropertyInfo(alias="nameTo")]
    """
    Filter for sales representatives whose `name` is alphabetically less than or
    equal to this value.
    """

    status: Literal["active", "all", "inactive"]
    """Filter for sales representatives that are active, inactive, or both."""

    updated_after: Annotated[str, PropertyInfo(alias="updatedAfter")]
    """
    Filter for sales representatives updated on or after this date and time, in ISO
    8601 format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the
    time is assumed to be 00:00:00 of that day.
    """

    updated_before: Annotated[str, PropertyInfo(alias="updatedBefore")]
    """
    Filter for sales representatives updated on or before this date and time, in ISO
    8601 format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the
    time is assumed to be 23:59:59 of that day.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["InventoryItemListParams"]


class InventoryItemListParams(TypedDict, total=False):
    conductor_end_user_id: Required[Annotated[str, PropertyInfo(alias="Conductor-End-User-Id")]]
    """
    The ID of the EndUser to receive this request (e.g.,
    `"Conductor-End-User-Id: {{END_USER_ID}}"`).
    """

    class_ids: Annotated[List[str], PropertyInfo(alias="classIds")]
    """Filter for inventory items of these classes.

    A class is a way end-users can categorize inventory items in QuickBooks.
    """

    cursor: str
    """
    The pagination token to fetch the next set of results when paginating with the
    `limit` parameter. Do not include this parameter on the first call. Use the
    `nextCursor` value returned in the previous response to request subsequent
    results.
    """

    full_names: Annotated[List[str], PropertyInfo(alias="fullNames")]
    """Filter for specific inventory items by their full-name(s), case-insensitive.

    Like `id`, `fullName` is a unique identifier for an inventory item, formed by by
    combining the names of its parent objects with its own `name`, separated by
    colons. For example, if an inventory item is under "Kitchen" and has the `name`
    "Cabinet", its `fullName` would be "Kitchen:Cabinet".

    **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
    query parameters for this request.

    **NOTE**: If any of the values you specify in this parameter are not found, the
    request will return an error.
    """

    ids: List[str]
    """
    Filter for specific inventory items by their QuickBooks-assigned unique
    identifier(s).

    **IMPORTANT**: If you include this parameter, QuickBooks will ignore all other
    query parameters for this request.

    **NOTE**: If any of the values you specify in this parameter are not found, the
    request will return an error.
    """

    limit: int
    """The maximum number of objects to return.

    Accepts values ranging from 1 to 150, defaults to 150. When used with
    cursor-based pagination, this parameter controls how many results are returned
    per page. To paginate through results, combine this with the `cursor` parameter.
    Each response will include a `nextCursor` value that can be passed to subsequent
    requests to retrieve the next page of results.
    """

    name_contains: Annotated[str, PropertyInfo(alias="nameContains")]
    """
    Filter for inventory items whose `name` contains this substring,
    case-insensitive. NOTE: If you use this parameter, you cannot also use
    `nameStartsWith` or `nameEndsWith`.
    """

    name_ends_with: Annotated[str, PropertyInfo(alias="nameEndsWith")]
    """
    Filter for inventory items whose `name` ends with this substring,
    case-insensitive. NOTE: If you use this parameter, you cannot also use
    `nameContains` or `nameStartsWith`.
    """

    name_from: Annotated[str, PropertyInfo(alias="nameFrom")]
    """
    Filter for inventory items whose `name` is alphabetically greater than or equal
    to this value.
    """

    name_starts_with: Annotated[str, PropertyInfo(alias="nameStartsWith")]
    """
    Filter for inventory items whose `name` starts with this substring,
    case-insensitive. NOTE: If you use this parameter, you cannot also use
    `nameContains` or `nameEndsWith`.
    """

    name_to: Annotated[str, PropertyInfo(alias="nameTo")]
    """
    Filter for inventory items whose `name` is alphabetically less than or equal to
    this value.
    """

    status: Literal["active", "all", "inactive"]
    """Filter for inventory items that are active, inactive, or both."""

    updated_after: Annotated[str, PropertyInfo(alias="updatedAfter")]
    """
    Filter for inventory items updated on or after this date and time, in ISO 8601
    format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time
    is assumed to be 00:00:00 of that day.

    **WARNING**: Due to a known issue in QuickBooks Desktop, the `updatedAfter`
    parameter may not correctly filter inventory items by their updated dates. To
    accurately retrieve the desired inventory items, we recommend avoiding this
    parameter and instead fetching a broader dataset, then filtering the results
    locally using the `updatedAt` property.
    """

    updated_before: Annotated[str, PropertyInfo(alias="updatedBefore")]
    """
    Filter for inventory items updated on or before this date and time, in ISO 8601
    format (YYYY-MM-DDTHH:mm:ss). If you only provide a date (YYYY-MM-DD), the time
    is assumed to be 23:59:59 of that day.

    **WARNING**: Due to a known issue in QuickBooks Desktop, the `updatedBefore`
    parameter may not correctly filter inventory items by their updated dates. To
    accurately retrieve the desired inventory items, we recommend avoiding this
    parameter and instead fetching a broader dataset, then filtering the results
    locally using the `updatedAt` property.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["DiscountItem", "Account", "Class", "CustomField", "Parent", "SalesTaxCode"]


class Account(BaseModel):
    id: Optional[str] = None
    """The unique identifier assigned by QuickBooks to this object.

    This ID is unique across all objects of the same type, but not across different
    QuickBooks object types.
    """

    full_name: Optional[str] = FieldInfo(alias="fullName", default=None)
    """
    The fully-qualified unique name for this object, formed by combining the names
    of its parent objects with its own `name`, separated by colons. Not
    case-sensitive.
    """


class Class(BaseModel):
    id: Optional[str] = None
    """The unique identifier assigned by QuickBooks to this object.

    This ID is unique across all objects of the same type, but not across different
    QuickBooks object types.
    """

    full_name: Optional[str] = FieldInfo(alias="fullName", default=None)
    """
    The fully-qualified unique name for this object, formed by combining the names
    of its parent objects with its own `name`, separated by colons. Not
    case-sensitive.
    """


class CustomField(BaseModel):
    name: str
    """The name of the custom field, unique for the specified `ownerId`.

    For public custom fields, this name is visible as a label in the QuickBooks UI.
    """

    owner_id: str = FieldInfo(alias="ownerId")
    """
    The identifier of the owner of the custom field, which QuickBooks internally
    calls a "data extension". For public custom fields visible in the UI, such as
    those added by the QuickBooks user, this is always "0". For private custom
    fields that are only visible to the application that created them, this is a
    valid GUID identifying the owning application. Internally, Conductor always
    fetches all public custom fields (those with an `ownerId` of "0") for all
    objects.
    """

    type: Literal[
        "amount_type",
        "date_time_type",
        "integer_type",
        "percent_type",
        "price_type",
        "quantity_type",
        "string_1024_type",
        "string_255_type",
    ]
    """The data type of this custom field."""

    value: str
    """The value of this custom field.

    The maximum length depends on the field's data type.
    """


class Parent(BaseModel):
    id: Optional[str] = None
    """The unique identifier assigned by QuickBooks to this object.

    This ID is unique across all objects of the same type, but not across different
    QuickBooks object types.
    """

    full_name: Optional[str] = FieldInfo(alias="fullName", default=None)
    """
    The fully-qualified unique name for this object, formed by combining the names
    of its parent objects with its own `name`, separated by colons. Not
    case-sensitive.
    """


class SalesTaxCode(BaseModel):
    id: Optional[str] = None
    """The unique identifier assigned by QuickBooks to this object.

    This ID is unique across all objects of the same type, but not across different
    QuickBooks object types.
    """

    full_name: Optional[str] = FieldInfo(alias="fullName", default=None)
    """
    The fully-qualified unique name for this object, formed by combining the names
    of its parent objects with its own `name`, separated by colons. Not
    case-sensitive.
    """


class DiscountItem(BaseModel):
    id: str
    """The unique identifier assigned by QuickBooks to this discount item.

    This ID is unique across all discount items but not across different QuickBooks
    object types.
    """

    account: Account
    """
    The posting account to which transactions involving this discount item are
    posted for tracking discounts.
    """

    barcode: Optional[str] = None
    """The discount item's barcode."""

    class_: Optional[Class] = FieldInfo(alias="class", default=None)
    """The discount item's class.

    Classes can be used to categorize objects into meaningful segments, such as
    department, location, or type of work. In QuickBooks, class tracking is off by
    default.
    """

    created_at: str = FieldInfo(alias="createdAt")
    """
    The date and time when this discount item was created, in ISO 8601 format
    (YYYY-MM-DDThh:mm:ss±hh:mm). The time zone is the same as the user's time zone
    in QuickBooks.
    """

    custom_fields: List[CustomField] = FieldInfo(alias="customFields")
    """
    The custom fields for the discount item object, added as user-defined data
    extensions, not included in the standard QuickBooks object.
    """

    description: Optional[str] = None
    """
    The discount item's description that will appear on sales forms that include
    this item.
    """

    discount_rate: Optional[str] = FieldInfo(alias="discountRate", default=None)
    """
    The monetary amount to subtract from the total or subtotal when applying this
    discount item to a transaction.

    **NOTE**: A flat rate discount applies to ALL lines recorded above it and
    distributes the discount amount equally across those lines, which affects tax
    calculations. For example, a $10 discount applied to a $100 taxable item and
    $100 non-taxable item would result in a $5 taxable discount and $5 non-taxable
    discount.
    """

    discount_rate_percent: Optional[str] = FieldInfo(alias="discountRatePercent", default=None)
    """
    The percentage amount to subtract from the total or subtotal when applying this
    discount item to a transaction.

    **NOTE**: A percentage discount only applies to the line immediately above it,
    so tax implications only affect that specific line.
    """

    external_id: Optional[str] = FieldInfo(alias="externalId", default=None)
    """
    A globally unique identifier (GUID) you, the developer, can provide for tracking
    this object in your external system. This field is immutable and can only be set
    during object creation.
    """

    full_name: str = FieldInfo(alias="fullName")
    """
    The case-insensitive fully-qualified unique name of this discount item, formed
    by combining the names of its hierarchical parent objects with its own `name`,
    separated by colons. For example, if a discount item is under "Discounts" and
    has the `name` "10% labor discount", its `fullName` would be "Discounts:10%
    labor discount".

    **NOTE**: Unlike `name`, `fullName` is guaranteed to be unique across all
    discount item objects. However, `fullName` can still be arbitrarily changed by
    the QuickBooks user when they modify the underlying `name` field.
    """

    is_active: bool = FieldInfo(alias="isActive")
    """Indicates whether this discount item is active.

    Inactive objects are typically hidden from views and reports in QuickBooks.
    Defaults to `true`.
    """

    name: str
    """The case-insensitive name of this discount item.

    Not guaranteed to be unique because it does not include the names of its
    hierarchical parent objects like `fullName` does. For example, two discount
    items could both have the `name` "10% labor discount", but they could have
    unique `fullName` values, such as "Discounts:10% labor discount" and
    "Promotions:10% labor discount".
    """

    object_type: Literal["qbd_discount_item"] = FieldInfo(alias="objectType")
    """The type of object. This value is always `"qbd_discount_item"`."""

    parent: Optional[Parent] = None
    """The parent discount item one level above this one in the hierarchy.

    For example, if this discount item has a `fullName` of "Discounts:10% labor
    discount", its parent has a `fullName` of "Discounts". If this discount item is
    at the top level, this field will be `null`.
    """

    revision_number: str = FieldInfo(alias="revisionNumber")
    """
    The current QuickBooks-assigned revision number of this discount item object,
    which changes each time the object is modified. When updating this object, you
    must provide the most recent `revisionNumber` to ensure you're working with the
    latest data; otherwise, the update will return an error.
    """

    sales_tax_code: Optional[SalesTaxCode] = FieldInfo(alias="salesTaxCode", default=None)
    """
    The default sales-tax code for this discount item, determining whether it is
    taxable or non-taxable. This can be overridden at the transaction-line level.

    Default codes include "Non" (non-taxable) and "Tax" (taxable), but custom codes
    can also be created in QuickBooks. If QuickBooks is not set up to charge sales
    tax (via the "Do You Charge Sales Tax?" preference), it will assign the default
    non-taxable code to all sales.
    """

    sublevel: float
    """The depth level of this discount item in the hierarchy.

    A top-level discount item has a `sublevel` of 0; each subsequent sublevel
    increases this number by 1. For example, a discount item with a `fullName` of
    "Discounts:10% labor discount" would have a `sublevel` of 1.
    """

    updated_at: str = FieldInfo(alias="updatedAt")
    """
    The date and time when this discount item was last updated, in ISO 8601 format
    (YYYY-MM-DDThh:mm:ss±hh:mm). The time zone is the same as the user's time zone
    in QuickBooks.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable
from datetime import date
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["InventoryAssemblyItemCreateParams", "Barcode", "Line"]


class InventoryAssemblyItemCreateParams(TypedDict, total=False):
    asset_account_id: Required[Annotated[str, PropertyInfo(alias="assetAccountId")]]
    """
    The asset account used to track the current value of this inventory assembly
    item in inventory.
    """

    cogs_account_id: Required[Annotated[str, PropertyInfo(alias="cogsAccountId")]]
    """
    The Cost of Goods Sold (COGS) account for this inventory assembly item, tracking
    the original direct costs of producing goods sold.
    """

    income_account_id: Required[Annotated[str, PropertyInfo(alias="incomeAccountId")]]
    """
    The income account used to track revenue from sales of this inventory assembly
    item.
    """

    name: Required[str]
    """The case-insensitive name of this inventory assembly item.

    Not guaranteed to be unique because it does not include the names of its
    hierarchical parent objects like `fullName` does. For example, two inventory
    assembly items could both have the `name` "Deluxe Kit", but they could have
    unique `fullName` values, such as "Assemblies:Deluxe Kit" and "Inventory:Deluxe
    Kit".

    Maximum length: 31 characters.
    """

    conductor_end_user_id: Required[Annotated[str, PropertyInfo(alias="Conductor-End-User-Id")]]
    """
    The ID of the EndUser to receive this request (e.g.,
    `"Conductor-End-User-Id: {{END_USER_ID}}"`).
    """

    barcode: Barcode
    """The inventory assembly item's barcode."""

    build_notification_threshold: Annotated[float, PropertyInfo(alias="buildNotificationThreshold")]
    """
    The inventory assembly item's minimum quantity threshold that triggers a build
    notification in QuickBooks. When the sum of `quantityOnHand` (current inventory)
    and `quantityOnOrder` (pending purchase orders) drops below this threshold,
    QuickBooks will notify users that more units need to be built or assembled. This
    helps ensure adequate inventory levels for inventory assembly items.
    """

    class_id: Annotated[str, PropertyInfo(alias="classId")]
    """The inventory assembly item's class.

    Classes can be used to categorize objects into meaningful segments, such as
    department, location, or type of work. In QuickBooks, class tracking is off by
    default.
    """

    external_id: Annotated[str, PropertyInfo(alias="externalId")]
    """
    A globally unique identifier (GUID) you, the developer, can provide for tracking
    this object in your external system. This field is immutable and can only be set
    during object creation.

    **IMPORTANT**: This field must be formatted as a valid GUID; otherwise,
    QuickBooks will return an error.
    """

    inventory_date: Annotated[Union[str, date], PropertyInfo(alias="inventoryDate", format="iso8601")]
    """
    The date when this inventory assembly item was converted into an inventory item
    from some other type of item, in ISO 8601 format (YYYY-MM-DD).
    """

    is_active: Annotated[bool, PropertyInfo(alias="isActive")]
    """Indicates whether this inventory assembly item is active.

    Inactive objects are typically hidden from views and reports in QuickBooks.
    Defaults to `true`.
    """

    lines: Iterable[Line]
    """The inventory assembly item's lines."""

    maximum_quantity_on_hand: Annotated[float, PropertyInfo(alias="maximumQuantityOnHand")]
    """The maximum quantity of this inventory assembly item desired in inventory."""

    parent_id: Annotated[str, PropertyInfo(alias="parentId")]
    """The parent inventory assembly item one level above this one in the hierarchy.

    For example, if this inventory assembly item has a `fullName` of
    "Assemblies:Deluxe Kit", its parent has a `fullName` of "Assemblies". If this
    inventory assembly item is at the top level, this field will be `null`.
    """

    preferred_vendor_id: Annotated[str, PropertyInfo(alias="preferredVendorId")]
    """
    The preferred vendor from whom this inventory assembly item is typically
    purchased.
    """

    purchase_cost: Annotated[str, PropertyInfo(alias="purchaseCost")]
    """
    The cost at which this inventory assembly item is purchased from vendors,
    represented as a decimal string.
    """

    purchase_description: Annotated[str, PropertyInfo(alias="purchaseDescription")]
    """
    The description of this inventory assembly item that appears on purchase forms
    (e.g., checks, bills, item receipts) when it is ordered or bought from vendors.
    """

    purchase_tax_code_id: Annotated[str, PropertyInfo(alias="purchaseTaxCodeId")]
    """The tax code applied to purchases of this inventory assembly item.

    Applicable in regions where purchase taxes are used, such as Canada or the UK.
    """

    quantity_on_hand: Annotated[float, PropertyInfo(alias="quantityOnHand")]
    """The current quantity of this inventory assembly item available in inventory.

    To change the `quantityOnHand` for an inventory assembly item, you must create
    an inventory-adjustment instead of updating this inventory assembly item
    directly.
    """

    sales_description: Annotated[str, PropertyInfo(alias="salesDescription")]
    """
    The description of this inventory assembly item that appears on sales forms
    (e.g., invoices, sales receipts) when sold to customers.
    """

    sales_price: Annotated[str, PropertyInfo(alias="salesPrice")]
    """
    The price at which this inventory assembly item is sold to customers,
    represented as a decimal string.
    """

    sales_tax_code_id: Annotated[str, PropertyInfo(alias="salesTaxCodeId")]
    """
    The default sales-tax code for this inventory assembly item, determining whether
    it is taxable or non-taxable. This can be overridden at the transaction-line
    level.

    Default codes include "Non" (non-taxable) and "Tax" (taxable), but custom codes
    can also be created in QuickBooks. If QuickBooks is not set up to charge sales
    tax (via the "Do You Charge Sales Tax?" preference), it will assign the default
    non-taxable code to all sales.
    """

    total_value: Annotated[str, PropertyInfo(alias="totalValue")]
    """The total value of this inventory assembly item.

    If `totalValue` is provided, `quantityOnHand` must also be provided and must be
    greater than zero. If both `quantityOnHand` and `purchaseCost` are provided,
    then `totalValue` will be set to `quantityOnHand` times `purchaseCost`,
    regardless of what `totalValue` is explicitly set to.
    """

    unit_of_measure_set_id: Annotated[str, PropertyInfo(alias="unitOfMeasureSetId")]
    """
    The unit-of-measure set associated with this inventory assembly item, which
    consists of a base unit and related units.
    """


class Barcode(TypedDict, total=False):
    allow_override: Annotated[bool, PropertyInfo(alias="allowOverride")]
    """Indicates whether to allow the barcode to be overridden."""

    assign_even_if_used: Annotated[bool, PropertyInfo(alias="assignEvenIfUsed")]
    """Indicates whether to assign the barcode even if it is already used."""

    value: str
    """The item's barcode value."""


class Line(TypedDict, total=False):
    inventory_item_id: Annotated[str, PropertyInfo(alias="inventoryItemId")]
    """The inventory item associated with this inventory assembly item line."""

    quantity: float
    """The quantity of the item associated with this inventory assembly item line.

    This field cannot be cleared.

    **NOTE**: Do not use this field if the associated item is a discount item.
    """
